/* $Id: AppController.h 9023 2006-09-16 11:15:14Z mindaugas $ */

/* AppController */

#import <Cocoa/Cocoa.h>
#import <ApplicationServices/ApplicationServices.h>
#include <sys/time.h>

#include <VFExtractor.h>
#include <VFMatcher.h>

#import "DBController.h"
#import "Options.h"
#import "Scanners.h"

struct record_t
{
	void * data;
	NSizeType size;
};

@interface AppController : NSObject
{
	struct timeval timeVal;
	
	struct record_t genRecords[10];
	int genRecordsCount;
	int genRecord;
	
	NSizeType maxRecordSize;
	
	NSString * gFingerID;
	
	int continuousScanIsStarted;
	int threadIsStarted;
	Scanners *scanners;
	pthread_t thread;
	
	HVFExtractor hExtractor;
	HVFMatcher hMatcher;
	HNFRecord hRecord;
	
	IBOutlet DBController *dBController;
	IBOutlet Options *options;
	
	IBOutlet NSApplication *app;

	IBOutlet NSWindow *mainWindow;
	IBOutlet NSWindowController *mainWindowController;
	
	IBOutlet NSImageView *firstImage;
	IBOutlet NSImageView *secondImage;
	IBOutlet NSScrollView *firstScrollView;
	IBOutlet NSScrollView *secondScrollView;
	
	IBOutlet NSTextView *logsTextView;
	
	IBOutlet NSTextField *fingerName;
	IBOutlet NSPanel *fingerNameWindow;
}

- (void)addLog:(NSString *)string retCode:(long)ret;

- (NSImage *)filterImage:(NSImage *)srcImage;

- (unsigned char *)getBitmapData:(NSImage *)image width:(int *)w height:(int *)h;
- (NSImage *)putBitmapData:(unsigned char *)data width:(int)w height:(int)h;

- (void)timerStart;
- (long)timerEnd;

- (NSString *)getFingerName;

- (IBAction)fingerNameOk:(id)sender;
- (void)findDuplicate:(struct record_t *)rec;

- (void)enroll:(NSImage *)image;
- (void)generalize:(NSImage *)image;
- (void)identify:(NSImage *)image;

- (IBAction)enrollFromFile:(id)sender;
- (IBAction)generalizeFromFile:(id)sender;
- (IBAction)identifyFromFile:(id)sender;

- (IBAction)enrollFromScanner:(id)sender;
- (IBAction)generalizeFromScanner:(id)sender;
- (IBAction)identifyFromScanner:(id)sender;

- (IBAction)continuousScanWithEnroll:(id)sender;
- (IBAction)continuousScanWithGeneralize:(id)sender;
- (IBAction)continuousScanWithIdentify:(id)sender;

- (IBAction)continuousScanStop:(id)sender;

- (IBAction)selectAES4000:(id)sender;
- (IBAction)selectAF_S2:(id)sender;
- (IBAction)selectMBF200:(id)sender;
- (IBAction)selectMBF200old:(id)sender;
- (IBAction)selectFM200:(id)sender;
- (IBAction)selectSHIMIZU:(id)sender;

- (IBAction)clearDB:(id)sender;
- (IBAction)openDB:(id)sender;
- (IBAction)saveDB:(id)sender;

- (HVFExtractor)getHExtractor;
- (HVFMatcher)getHMatcher;

// ========

- (int) getContinuousScanIsStarted;
- (void) setContinuousScanIsStarted:(int) value;

- (int) getThreadIsStarted;
- (void) setThreadIsStarted:(int) value;

- (Scanners *) getScanners;

// ============

- (void) startMultithreadingMode:(id)sender;

@end
