/* $Id: Scanners.m 9016 2006-09-15 16:07:33Z mindaugas $ */

#import "Scanners.h"

struct scanner_info * scanners[] = 
{
	&scanner_aes4000, // AES 4000
	&scanner_af_s2, // AF-S2
	&scanner_mbf200, // Fujitsu/MBF200
	&scanner_mbf200old, // MBF200
	&scanner_fm200, // FM200 Scanner
	&scanner_shimizu // Shimizu/Tacoma Optical Capture Device
};

#define NUMBER_OF_SCANNERS 6

@implementation Scanners

- (void)init:(int)index
{
	scanner = scanners[index];
	
	isInit = 0;
}

- (int)getNumberOfScanners
{
	return NUMBER_OF_SCANNERS;
}

- (int)selectScanner:(unsigned int)index
{
	if (index >= NUMBER_OF_SCANNERS)
	{
		return -1;
	}
	
	if (isInit)
	{
		return -1;
	}
	
	scanner = scanners[index];
}

- (NSString *)getScannerName
{
	if (scanner)
	{
		return [NSString stringWithCString:scanner->name];
	}
	else
	{
		return @"";
	}
}

- (int)initScanner
{
	int ret;
	
	isInit = 0;

	if (!scanner)
	{
		return -1;
	}
	
	if (isInit)
	{
		return -1;
	}
	
	if (ret = scanner->init())
	{
		return ret;
	}
	isInit = 1;
	
	return 0;
}

- (int)isInit
{
	return isInit;
}

- (NSImage *)readFromScanner
{
	int w = 0;
	int h = 0;
	int x, y;

	unsigned char * srcData = NULL;

	unsigned char *destData = NULL;
	NSImage * destImage = NULL;
	NSBitmapImageRep * destImageRep = NULL;

	if (!isInit)
	{
		return NULL;
	}

	srcData = scanner->read(&w, &h);

	if (!srcData)
	{
		return NULL;
	}

	destImage = [[NSImage alloc] initWithSize:NSMakeSize(w, h)];

	destImageRep = [[[NSBitmapImageRep alloc] 
		initWithBitmapDataPlanes:NULL
		pixelsWide:w 
		pixelsHigh:h 
		bitsPerSample:8 
		samplesPerPixel:1
		hasAlpha:NO
		isPlanar:NO
		colorSpaceName:NSCalibratedWhiteColorSpace
		bytesPerRow:w 
		bitsPerPixel:8] autorelease];

	destData = [destImageRep bitmapData];

	memcpy(destData, srcData, w*h);

	[destImage addRepresentation:destImageRep];
	return destImage;
}

- (void)closeScanner
{
	if (scanner)
	{
		if (isInit)
		{
			scanner->close();
			isInit = 0;
		}
	}
}

@end
