#include "System.h"
#include "Dialogs.h"
#include "Utils.h"
#include <ShlObj.h>

/////////////////
//             //
// CFileDialog //
//             //
/////////////////

const int max_filename_length = 32768;

CFileDialog::CFileDialog()
{
	Reset();
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.nMaxFile = max_filename_length;
}

void CFileDialog::BeforeShow(CForm *owner)
{
	ofn.hwndOwner = owner ? owner->Handle() : NULL;
	if(Filter.IsEmpty()) ofn.lpstrFilter = NULL;
	else
	{
		int l = Filter.Length();
		TCHAR *s = new TCHAR[l + 2];
		ofn.lpstrFilter = s;
		Filter.Copy(s);
		for(int i = 0; i < l; i++, s++)
			if(*s == TEXT('|')) *s = TEXT('\0');
		*(s + 1) = TEXT('\0');
	}
	ofn.nFilterIndex = FilterIndex + 1;
	ofn.lpstrInitialDir = InitialDirectory.IsEmpty() ? NULL : (LPCTSTR)InitialDirectory;
	ofn.lpstrTitle = Title.IsEmpty() ? NULL : (LPCTSTR)Title;
	ofn.Flags = OFN_HIDEREADONLY | OFN_EXPLORER;
	if(AllowMultiSelect) ofn.Flags |= OFN_ALLOWMULTISELECT;
	if(CreatePrompt) ofn.Flags |= OFN_CREATEPROMPT;
	if(EnableSizing) ofn.Flags |= OFN_ENABLESIZING;
	if(FileMustExist) ofn.Flags |= OFN_FILEMUSTEXIST;
	if(NoChangeDirectory) ofn.Flags |= OFN_NOCHANGEDIR;
	if(OverwritePrompt) ofn.Flags |= OFN_OVERWRITEPROMPT;
	if(PathMustExist) ofn.Flags |= OFN_PATHMUSTEXIST;
	ofn.lpstrDefExt = DefaultExtension.IsEmpty() ? NULL : DefaultExtension;
	ofn.lpstrFile = new TCHAR[max_filename_length];
	FileName.Copy(ofn.lpstrFile);
	FileNames.Clear();
}

void CFileDialog::AfterShow()
{
	if(ofn.lpstrFilter) delete[] (TCHAR*)ofn.lpstrFilter;
	FilterIndex = ofn.nFilterIndex - 1;
	int l = _tcslen(ofn.lpstrFile);
	if(!AllowMultiSelect || ofn.lpstrFile[l + 1] == 0)
	{
		FileName.Assign(ofn.lpstrFile);
		FileNames.Add(FileName);
	}
	else
	{
		string dir(ofn.lpstrFile);
		string file;
		TCHAR *s = ofn.lpstrFile + l + 1;
		while(*s)
		{
			l = _tcslen(s);
			FileNames.Add(MakeFullFileName(dir, string(s)));
			s += l + 1;
		}
		FileName = FileNames[0];
	}
	/*if(!AllowMultiSelect && FileNameHasNoExtension(FileName) && !IsStringEmpty(DefaultExtension))
	{
		FileName = ChangeFileExtension(FileName, DefaultExtension);
		FileNames[0] = FileName;
	}*/
	delete[] ofn.lpstrFile;
}

void CFileDialog::Reset()
{
	Title.Clear();
	FileName.Clear();
	Filter.Clear();
	FilterIndex = -1;
	InitialDirectory.Clear();
	DefaultExtension.Clear();
	AllowMultiSelect = false;
	CreatePrompt = false;
	EnableSizing = true;
	NoChangeDirectory = true;
	FileNames.Clear();
}

/////////////////////
//                 //
// COpenFileDialog //
//                 //
/////////////////////

void COpenFileDialog::Reset()
{
	CFileDialog::Reset();
	FileMustExist = true;
	PathMustExist = true;
	OverwritePrompt = false;
}

int COpenFileDialog::Show(CForm *owner)
{
	BeforeShow(owner);
	if(GetOpenFileName(&ofn))
	{
		AfterShow();
		return IDOK;
	}
	return IDCANCEL;
}

/////////////////////
//                 //
// CSaveFileDialog //
//                 //
/////////////////////

void CSaveFileDialog::Reset()
{
	CFileDialog::Reset();
	FileMustExist = false;
	PathMustExist = false;
	OverwritePrompt = true;
}

int CSaveFileDialog::Show(CForm *owner)
{
	BeforeShow(owner);
	if(GetSaveFileName(&ofn))
	{
		AfterShow();
		return IDOK;
	}
	return IDCANCEL;
}

////////////////////////
//                    //
// BrowseForFolderBox //
//                    //
////////////////////////

#ifndef BIF_NEWDIALOGSTYLE
#define BIF_NEWDIALOGSTYLE 0x0040
#endif
#ifndef BIF_NONEWFOLDERBUTTON
#define BIF_NONEWFOLDERBUTTON 0x0200
#endif

bool BrowseForFolderBox(CForm *owner, const string &title, string &path, bool new_folder, bool type_folder)
{
	BROWSEINFO bi;
	memset(&bi, 0, sizeof(bi));
	bi.hwndOwner = owner ? owner->Handle() : NULL;
	bi.lpszTitle = title;
	bi.ulFlags = BIF_NEWDIALOGSTYLE | (type_folder ? BIF_EDITBOX : 0) | (new_folder ? 0 : BIF_NONEWFOLDERBUTTON);
	LPITEMIDLIST iidl = SHBrowseForFolder(&bi);
	if(iidl)
	{
		path.SetLength(1000);
		bool r = SHGetPathFromIDList(iidl, path) != 0;
		path.Update();
		return r;
	}
	else return false;
}

//////////////////
//              //
// CColorDialog //
//              //
//////////////////

CColorDialog::CColorDialog()
{
	Reset();
	memset(&cl, 0, sizeof(cl));
	cl.lStructSize = sizeof(cl);
	cl.lpCustColors = CustomColors;
	cl.Flags = CC_RGBINIT | CC_ANYCOLOR;
}

void CColorDialog::Reset()
{
	Color = RGB(0, 0, 0);
}

int CColorDialog::Show(CForm * owner)
{
	cl.hwndOwner = owner ? owner->Handle() : NULL;
	cl.rgbResult = Color;
	bool result = ChooseColor(&cl) != 0;
	cl.hwndOwner = NULL;
	Color = cl.rgbResult;
	return result ? IDOK : IDCANCEL;
}

////////////////
//            //
// CInputForm //
//            //
////////////////

CInputForm::CInputForm()
	: CForm()
{
	SetMinimizeBox(false);
	SetMaximizeBox(false);
	SetFormBorder(fbDialog);
	SetFormPosition(fpOwnerCenter);
	
	int l = 150;
	
	labelPrompt = new CLabel();
	labelPrompt->SetLocation(8, 8);
	labelPrompt->SetWidth(l);
	editValue = new CEdit();
	editValue->SetLocation(labelPrompt->Left(), labelPrompt->Bottom() + 8);
	editValue->SetWidth(l);
	buttonOK = new CButton();
	buttonOK->SetText(STRING("OK"));
	buttonOK->SetModalResult(IDOK);
	buttonOK->SetDefault(true);
	buttonOK->SetTop(editValue->Bottom() + 8);
	buttonCancel = new CButton();
	buttonCancel->SetText(STRING("Cancel"));
	buttonCancel->SetModalResult(IDCANCEL);
	buttonCancel->SetCancel(true);
	buttonCancel->SetTop(buttonOK->Top());
	buttonCancel->SetRight(editValue->Right());
	buttonOK->SetRight(buttonCancel->Left() - 8);
	SetClientSize(buttonCancel->Right() + 8, buttonCancel->Bottom() + 8);

	AddControl(labelPrompt);
	AddControl(editValue);
	AddControl(buttonOK);
	AddControl(buttonCancel);

	SetActiveControl(editValue);
}

const string & CInputForm::Prompt()
{
	return labelPrompt->Text();
}

void CInputForm::SetPrompt(const string &value)
{
	labelPrompt->SetText(value);
}

const string & CInputForm::Value()
{
	return editValue->Text();
}

void CInputForm::SetValue(const string &value)
{
	editValue->SetText(value);
}

bool InputBox(CForm * owner, const string &title, const string &prompt, string &value)
{
	CInputForm * form = new CInputForm();
	form->SetText(title);
	form->SetPrompt(prompt);
	form->SetValue(value);
	form->SetOwner(owner);
	bool ok = form->ShowModal() == IDOK;
	if(ok) value = form->Value();
	return ok;
}
