#pragma once

#include "Controls.h"
#include "StdControls.h"
#include "Dialogs.h"
#include "FileListForm.rh"
#include "VFFileList.h"

class CFileListForm: public CForm
{
private:
	VFFileList *filelist;
	bool modified;
	int resolution;
	string finger_id;

	COpenFileDialog *open_dialog;
	CSaveFileDialog *save_dialog;
	COpenFileDialog *add_dialog;

	CListView *listview;
	CStatusBar *statusbar;

	bool changing;

	void OnFileNew();
	void OnFileOpen();
	bool OnFileSave();
	void OnFileAdd();
	void OnFileExit();
	void OnEditAdd();
	void OnEditAddDirectory();
	void OnEditNewEntry();
	void OnEditEditEntry();
	void OnEditDelete();
	void OnEditMoveUp();
	void OnEditMoveDown();
	void OnEditClearFingerID();
	void OnEditSelectAll();
	void OnEditSelectNone();
	void OnEditGoTo();

	int AddFromDirectory(const string &path, const string &filter);
	void UpdateSelection();
	bool ClearPrompt();
	bool OnCloseQuery();

	void UpdateFileList();

	void SetFile(int index, const string &filename)
	{
		VFFileListItem &item = (*filelist)[index];
		item.filename = filename;
		item.resolution = resolution;
		item.finger_id = finger_id;
	}

	void AddFile(const string &filename)
	{
		VFFileListItem item(filename, resolution, MakeFingerID(finger_id, filename, false));
		filelist->Add(item);
		listview->AddItem();
	}

protected:
	LPCTSTR GetMenuName() { return MAKEINTRESOURCE(IDM_FILELIST); }
	LPCTSTR GetAccelName() { return MAKEINTRESOURCE(IDA_FILELIST); }

	void OnCreate();
	void OnCommand(WORD id, WORD event, CControl * control);
	LRESULT OnNotify(LPNMHDR nmh, CControl * control);
	LRESULT OnSize(DWORD resizingType, WORD clientWidth, WORD clientHeight);

public:
	CFileListForm();
	~CFileListForm();

	VFFileList * FileList() { return filelist; }
	int Resolution() { return resolution; }
	const string& FingerID() { return finger_id; }

	void SetFileList(VFFileList *value);
	void SetResolution(int value);
	void SetFingerID(const string &value);
};

void FileListBox(CForm * owner);
