#pragma once

#include "Streams.h"

enum LogMode {lmAppend, lmOverwrite};

struct LogState
{
	string name;
	bool asIndex;
	vector<int> counts;
	int count;
	vector<int> countsStarted;
	vector<int> progresses;
	int progress;
	float progressP;
	vector<double> times;
	double time;
	vector<double> timesEstimated;
	double timeEstimated;
	vector<double> timesRemaining;
	double timeRemaining;

	LogState()
	{
	}

	LogState(const string &name)
	{
		this->name = name;
		this->asIndex = false;
		count = 0;
		progress = 0;
		progressP = 0.0f;
		time = 0.0;
		timeEstimated = 0.0;
		timeRemaining = 0.0;
	}
};

class CLog
{
private:
	CRITICAL_SECTION cs;
	
	CFileStream fileStream;

	string lastLine;

	vector<LogState> states;

	void AddLine(const string &text);
	void Add(const string &text);

	void CheckStatesEmpty();
	void SetStateProgressRaw(LogState &state, const vector<int> &progresses);
	void SetStateTimeRaw(LogState &state, const vector<double> &times);
	void UpdateStateTime();

protected:
	virtual void OnClear();
	virtual void OnAddLine(const string &text);
	virtual void OnAdd(const string &text);
	virtual void OnUpdateState(const vector<LogState> &states);

	void UpdateState();

public:
	CLog();
	virtual ~CLog();

	void OpenFile(const string &fileName, LogMode mode = lmOverwrite);
	void CloseFile();

	void Flush();

	void WriteLine(const TCHAR *format, ...);
	void WriteLine(DWORD id, ...);
	void Write(const TCHAR *format, ...);
	void Write(DWORD id, ...);
	void NewLine();

	void Clear();

	string LastLine();

	void AddState(const string &value);
	void RemoveState();
	void InitStateProgressS(const vector<int> &counts, const vector<int> &countsStarted, bool asIndex = false);
	void InitStateProgressS(int count, int countStarted, bool asIndex = false)
	{
		vector<int> counts;
		counts.Add(count);
		vector<int> countsStarted;
		countsStarted.Add(countStarted);
		InitStateProgressS(counts, countsStarted, asIndex);
	}
	void InitStateProgressS(int count1, int count2, int countStarted1, int countStarted2, bool asIndex = false)
	{
		vector<int> counts;
		counts.Add(count1);
		counts.Add(count2);
		vector<int> countsStarted;
		countsStarted.Add(countStarted1);
		countsStarted.Add(countStarted2);
		InitStateProgressS(counts, countsStarted, asIndex);
	}
	void InitStateProgress(const vector<int> &counts, bool asIndex = false)
	{
		vector<int> countsStarted(counts.Length());
		countsStarted.Fill(0);
		InitStateProgressS(counts, countsStarted, asIndex);
	}
	void InitStateProgress(int count, bool asIndex = false)
	{
		vector<int> counts;
		counts.Add(count);
		InitStateProgress(counts, asIndex);
	}
	void InitStateProgress(int count1, int count2, bool asIndex = false)
	{
		vector<int> counts;
		counts.Add(count1);
		counts.Add(count2);
		InitStateProgress(counts, asIndex);
	}
	void SetStateProgress(const vector<int> &progresses);
	void SetStateProgress(int progress)
	{
		vector<int> progresses;
		progresses.Add(progress);
		SetStateProgress(progresses);
	}
	void SetStateProgress(int progress1, int progress2)
	{
		vector<int> progresses;
		progresses.Add(progress1);
		progresses.Add(progress2);
		SetStateProgress(progresses);
	}
	void SetStateTime(const vector<double> &times);
	void SetStateTime(double time)
	{
		vector<double> times;
		times.Add(time);
		SetStateTime(times);
	}
	void SetStateTime(double time1, double time2)
	{
		vector<double> times;
		times.Add(time1);
		times.Add(time2);
		SetStateTime(times);
	}
	void SetStateProgressAndTime(const vector<int> &progresses, const vector<double> &times);
	void SetStateProgressAndTime(int progress, double time)
	{
		vector<int> progresses;
		progresses.Add(progress);
		vector<double> times;
		times.Add(time);
		SetStateProgressAndTime(progresses, times);
	}
	void SetStateProgressAndTime(int progress1, int progress2, double time1, double time2)
	{
		vector<int> progresses;
		progresses.Add(progress1);
		progresses.Add(progress2);
		vector<double> times;
		times.Add(time1);
		times.Add(time2);
		SetStateProgressAndTime(progresses, times);
	}
};
