#include "System.h"
#include "VFFeatures.h"

CVFFeatures::CVFFeatures()
	: data(NULL), dataLength(0), G(255)
{
}

CVFFeatures::CVFFeatures(const BYTE * data, DWORD dataLength)
	: data(NULL), dataLength(0)
{
	SetData(data, dataLength);
}

CVFFeatures::~CVFFeatures()
{
	Clear();
}

void CVFFeatures::Assign(const CVFFeatures * features)
{
	SetData(features->data, features->dataLength);
}

void CVFFeatures::Clear()
{
	delete[] data; data = NULL;
	dataLength = 0;
	G = 255;
	Minutiae.Clear();
	SingularPoints.Clear();
	BlockedOrientations.Clear();
}

void CVFFeatures::SetData(const BYTE * data, DWORD dataLength)
{
	Clear();
	if(data)
	{
		this->data = new BYTE[dataLength];
		memcpy(this->data, data, dataLength);
		this->dataLength = dataLength;
	}

	int result;

	G = (BYTE)(result = VFFeatGetG(data)); VFCheckResult(result);

	int mCount = result = VFFeatGetMinutiaCount(data); VFCheckResult(result);
	Minutiae.SetLength(mCount);
	result = VFFeatGetMinutiae(data, Minutiae.Data()); VFCheckResult(result);

	int spCount = result = VFFeatGetSPCount(data); VFCheckResult(result);
	SingularPoints.SetLength(spCount);
	result = VFFeatGetSP(data, SingularPoints.Data()); VFCheckResult(result);

	int boWidth, boHeight;
	result = VFFeatGetBOSize(data, &boWidth, &boHeight); VFCheckResult(result);
	BlockedOrientations.SetSize(boWidth, boHeight);
	result = VFFeatGetBO(data, BlockedOrientations.Bits()); VFCheckResult(result);
}

DWORD CVFFeatures::GetData(BYTE * data)
{
	memcpy(data, this->data, dataLength);
	return dataLength;
}
