#pragma once

class CVFImage
{
private:
	int width;
	int height;
	DWORD size;
	BYTE * bits;

	void CheckPixelIndex(int x, int y)
	{
		if(x < 0 || x >= width || y < 0 || y >= height)
			throw exception("Pixel index out of bounds");
	}

public:
	CVFImage();
	CVFImage(int width, int height);
	CVFImage(const string &fileName);
	~CVFImage();

	bool IsEmpty() { return bits == NULL; }
	void Clear();

	BYTE *Bits() { return bits; }
	int Width() { return width; }
	int Height() { return height; }
	void SetWidth(int value) { SetSize(value, height); }
	void SetHeight(int value) { SetSize(width, value); }
	void SetSize(int width, int height);

	BYTE GetPixel(int x, int y);
	void SetPixel(int x, int y, BYTE value);

	void Assign(const CVFImage * image);
	void Assign(int width, int height, const BYTE * bits);

	bool LoadFromFile(const string &filename);
	bool SaveToFile(const string &filename, int format_index = -1);

	HBITMAP GetBitmap(COLORREF color_min = RGB(0, 0, 0), COLORREF color_max = RGB(255, 255, 255));
	HBITMAP GetBitmap(int bitmapWidth, int bitmapHeight, COLORREF color_min = RGB(0, 0, 0), COLORREF color_max = RGB(255, 255, 255));
};

class CVFImageFormat
{
public:
	CVFImageFormat();
	~CVFImageFormat();

	virtual bool CanLoad() = 0;
	virtual bool CanSave() = 0;
	virtual string FileFilter() = 0;
	virtual string FileFilterName() = 0;
	string GetFileFilterString(bool filter_in_name = true);
	virtual bool Load(CVFImage *image, const string &filename) = 0;
	virtual bool Save(CVFImage *image, const string &filename) = 0;
};

typedef vector<CVFImageFormat*> CVFImageFormats;

class CVFImageIO
{
private:
	static CVFImageFormats formats;

	static void AddFormat(CVFImageFormat *format);
	static void RemoveFormat(CVFImageFormat *format);

public:
	static int FormatCount();
	static CVFImageFormat* Format(int index);

	static bool CanLoad();
	static bool CanSave();

	static string LoadFileFilter();
	static string SaveFileFilter();
	static string GetLoadFileFilterString(bool filter_in_name = true, bool all_filters = true, bool all_files = true);
	static string GetSaveFileFilterString(bool filter_in_name = true);

	static bool Load(CVFImage *image, const string &filename);
	static bool Save(CVFImage *image, string filename, int format_index = -1);

	friend class CVFImageFormat;
};
