#include "System.h"
#include "VFImageFormatBMP.h"

CVFImageFormatBMP image_format_bmp;

bool CVFImageFormatBMP::CanLoad()
{
	return true;
}

bool CVFImageFormatBMP::CanSave()
{
	return true;
}

string CVFImageFormatBMP::FileFilter()
{
	return TEXT("*.bmp");
}

string CVFImageFormatBMP::FileFilterName()
{
	return TEXT("Bitmap files");
}

inline int GetLeftShiftCount(DWORD dwVal)
{
	int nCount = 0;
	for(int i = 0; i < sizeof(DWORD) * 8; i++, dwVal >>= 1)
		if(dwVal & 1) nCount++;
	return 8 - nCount;
}

inline int GetRightShiftCount(DWORD dwVal)
{
	for(int i = 0; i < sizeof(DWORD) * 8; i++, dwVal >>= 1)
		if(dwVal & 1) return i;
	return -1;
}

#define BITMAPFILESIGNATURE 0x4D42

bool CVFImageFormatBMP::Load(CVFImage *image, const string &filename)
{
	HANDLE bmp = LoadImage(NULL, filename, IMAGE_BITMAP, 0, 0,
		LR_DEFAULTCOLOR | LR_CREATEDIBSECTION | LR_DEFAULTSIZE | LR_LOADFROMFILE);
	if(bmp == NULL) return false;
	DIBSECTION ds;
	if(!GetObject(bmp, sizeof(ds), &ds))
	{
		DeleteObject(bmp);
		return false;
	}
	image->SetSize(ds.dsBm.bmWidth, ds.dsBm.bmHeight);
	BYTE *bits = image->Bits();
	int x, y;
	DWORD rmask, gmask, bmask;
	int rright, gright, bright;
	int rleft, gleft, bleft;
	BITMAPINFO bmi;
	WORD wColor;
	WORD* pwBits;
	DWORD* pdwBits;
	DWORD dwColor;
	BYTE* pbBits, *pBuffer, r, g, b;
	HDC hdc;
	int nPad = ds.dsBm.bmWidthBytes - (((ds.dsBmih.biWidth * ds.dsBmih.biBitCount) + 7) / 8);
	int p = (image->Height() - 1) * image->Width(), pp;

	switch(ds.dsBmih.biBitCount)
	{
		case 1: case 4: case 8:
			hdc = GetDC(NULL);
			pBuffer = new BYTE[ds.dsBmih.biWidth * 3 + 4];

			memset(&bmi, 0, sizeof(bmi));
			bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
			bmi.bmiHeader.biWidth = ds.dsBmih.biWidth;
			bmi.bmiHeader.biHeight = ds.dsBmih.biHeight;
			bmi.bmiHeader.biPlanes = 1;
			bmi.bmiHeader.biBitCount = 24;
			bmi.bmiHeader.biCompression = BI_RGB;

			for(y = 0; y < ds.dsBmih.biHeight; y++, p -= image->Width())
			{
				GetDIBits(hdc, (HBITMAP)bmp, y, 1, pBuffer, &bmi, DIB_RGB_COLORS);
				pbBits = pBuffer;
				for(x = 0, pp = p; x < ds.dsBmih.biWidth; x++, pp++)
				{
					b = *pbBits++;
					g = *pbBits++;
					r = *pbBits++;
					bits[pp] = (BYTE)((b + r + g) / 3);
				}
			}

			delete pBuffer;
			ReleaseDC(NULL, hdc);
			break;

		case 16:
			if(ds.dsBmih.biCompression == BI_BITFIELDS) 
			{
				rmask = ds.dsBitfields[0];
				gmask = ds.dsBitfields[1];
				bmask = ds.dsBitfields[2];
			}
			else 
			{
				rmask = 0x7C00;
				gmask = 0x03E0;
				bmask = 0x001F;
			}

			rright = GetRightShiftCount(rmask);
			gright = GetRightShiftCount(gmask);
			bright = GetRightShiftCount(bmask);

			rleft = GetLeftShiftCount(rmask);
			gleft = GetLeftShiftCount(gmask);
			bleft = GetLeftShiftCount(bmask);

			pwBits = (WORD*)ds.dsBm.bmBits;
			for(y = 0; y < ds.dsBmih.biHeight; y++, p -= image->Width())
			{
				for(x = 0, pp = p; x < ds.dsBmih.biWidth; x++, pp++)
				{
					wColor = *pwBits++;
					b = (BYTE)(((wColor & (WORD) bmask) >> bright) << bleft);
					g = (BYTE)(((wColor & (WORD) gmask) >> gright) << gleft);
					r = (BYTE)(((wColor & (WORD) rmask) >> rright) << rleft);
					bits[pp] = (BYTE)((b + r + g) / 3);
				}
				pwBits = (WORD*)(((BYTE*)pwBits) + nPad);
			}
			break;

		case 24:
			pbBits = (BYTE*)ds.dsBm.bmBits;
			for(y = 0; y < ds.dsBmih.biHeight; y++, p -= image->Width())
			{
				for(x = 0, pp = p; x < ds.dsBmih.biWidth; x++, pp++)
				{
					b = *pbBits++;
					g = *pbBits++;
					r = *pbBits++;
					bits[pp] = (BYTE)((b + r + g) / 3);
				}
				pbBits += nPad;
			}
			break;

		case 32:
			if(ds.dsBmih.biCompression == BI_BITFIELDS) 
			{
				rmask = ds.dsBitfields[0];
				gmask = ds.dsBitfields[1];
				bmask = ds.dsBitfields[2];
			}
			else 
			{
				rmask = 0x00FF0000;
				gmask = 0x0000FF00;
				bmask = 0x000000FF;
			}

			rright = GetRightShiftCount(rmask);
			gright = GetRightShiftCount(gmask);
			bright = GetRightShiftCount(bmask);

			pdwBits = (DWORD*)ds.dsBm.bmBits;
			for(y = 0; y < ds.dsBmih.biHeight; y++, p -= image->Width())
			{
				for (x = 0, pp = p; x < ds.dsBmih.biWidth; x++, pp++)
				{
					dwColor = *pdwBits++;
					b = (BYTE)((dwColor & bmask) >> bright);
					g = (BYTE)((dwColor & gmask) >> gright);
					r = (BYTE)((dwColor & rmask) >> rright);
					bits[pp] = (BYTE)((b + r + g) / 3);
				}
				pdwBits = (DWORD*)(((BYTE*) pdwBits) + nPad);
			}
			break;

	default:
		DeleteObject(bmp);
		image->Clear();
		return false;
	}
	DeleteObject(bmp);
	return true;
}

bool CVFImageFormatBMP::Save(CVFImage *image, const string &filename)
{
	BITMAPFILEHEADER bfh;
	BITMAPINFOHEADER bih;
	DWORD width_bytes;

	bfh.bfType = BITMAPFILESIGNATURE;
	bfh.bfReserved1 = 0;
	bfh.bfReserved2 = 0;
	bfh.bfOffBits = sizeof(bfh) + sizeof(bih) + sizeof(RGBQUAD) * 256;

	bih.biSize = sizeof(bih);
	bih.biWidth = image->Width();
	bih.biHeight = image->Height();
	bih.biPlanes = 1;
	bih.biBitCount = 8;
	bih.biCompression = BI_RGB;
	width_bytes = (image->Width() / 4 + (DWORD)(image->Width() % 4 ? 1 : 0)) * 4;
	bih.biSizeImage = width_bytes * image->Height();
	bih.biXPelsPerMeter = 0;
	bih.biYPelsPerMeter = 0;
	bih.biClrUsed = 0;
	bih.biClrImportant = 0;

	bfh.bfSize = bfh.bfOffBits + bih.biSizeImage;

	FILE *f = _tfopen(filename, TEXT("wb"));

	if(fwrite(&bfh, sizeof(bfh), 1, f) != 1) { fclose(f); return false; }
	if(fwrite(&bih, sizeof(bih), 1, f) != 1) { fclose(f); return false; }

	RGBQUAD color;
	int i;
	color.rgbReserved = 0;
	for(i = 0; i < 256; i++)
	{
		color.rgbBlue = (BYTE)i;
		color.rgbGreen = (BYTE)i;
		color.rgbRed = (BYTE)i;
		if(fwrite(&color, sizeof(color), 1, f) != 1) { fclose(f); return false; }
	}
	
	width_bytes -= image->Width();
	BYTE *bits = image->Bits() + (image->Height() - 1) * image->Width();
	for(i = 0; i < image->Height(); i++, bits -= image->Width())
	{
		if(fwrite(bits, image->Width(), 1, f) != 1) { fclose(f); return false; }
		if(fseek(f, width_bytes, SEEK_CUR) != 0) { fclose(f); return false; }
	}
	fclose(f);
	return true;
}
