#pragma once

// These operators are defined in "new" header in Microsoft Visual C++ 6.0
// and in "new.h" header in Microsoft Visual C++ 7.0.
// To provide compability with both compilers (and with others) place them here
#ifndef __PLACEMENT_NEW_INLINE
#define __PLACEMENT_NEW_INLINE
inline void * __cdecl operator new(size_t, void * p) { return p; }
#if _MSC_VER >= 1200
inline void __cdecl operator delete(void *, void *) {}
#endif
#endif


#define vector_AlignLength(l, a) ((l / a + (l % a ? 1 : 0)) * a)
#define vector_AlignCapacity(l, m, a) l <= m ? vector_AlignLength(l, a) :

template<class T> class vector
{
private:
	T * items, * end;
	int length;
	int capacity;

	static void CopyItems(T * dst, const T * src, int l)
	{
		memcpy(dst, src, l * sizeof(T));
	}

	static void MoveItems(T * dst, T * src, int l)
	{
		memmove(dst, src, l * sizeof(T));
	}

	void CheckIndex(int index, bool end = false) const
	{
		if(index < 0 || (end && index > length) || (!end && index >= length))
			throw exception("Vector index out of bounds");
	}

	typedef vector<T> InstanceType;

public:
	vector()
		: items(NULL), capacity(0), end(NULL), length(0)
	{
	}

	explicit vector(int l)
		: items(NULL), capacity(0), end(NULL), length(0)
	{
		SetLength(l);
	}

	vector(int l, const T &item)
		: items(NULL), capacity(0), end(NULL), length(0)
	{
		Assign(l, item);
	}

	vector(const InstanceType &v)
		: items(NULL), capacity(0), end(NULL), length(0)
	{
		Assign(v);
	}

	~vector()
	{
		for(T * i = items; i < end; i++) i->~T();
		free(items);
	}

	bool IsEmpty() const
	{
		return length == 0;
	}

	int Length() const
	{
		return length;
	}

	void SetLength(int value)
	{
		T * cur = items + value;
		for(; cur < end; cur++) cur->~T();
		SetCapacity(value);
		cur = items + value;
		for(T * i = end; i < cur; i++) new(i) T;
		length = value;
		end = cur;
	}

	int Capacity() const
	{
		return capacity;
	}

	void SetCapacity(int value)
	{
		int c = vector_AlignCapacity(value, 8, 1)
			vector_AlignCapacity(value, 16, 2)
			vector_AlignCapacity(value, 32, 4)
			vector_AlignCapacity(value, 64, 8)
			vector_AlignCapacity(value, 128, 16)
			vector_AlignCapacity(value, 256, 32)
			vector_AlignCapacity(value, 512, 64)
			vector_AlignCapacity(value, 1024, 128)
			vector_AlignCapacity(value, 2048, 256)
			vector_AlignCapacity(value, 4096, 512)
			vector_AlignLength(value, 1024);
		if(c != capacity)
		{
			items = (T*)realloc(items, c * sizeof(T));
			if(c && !items) throw exception("Out of memory in vector");
			capacity = c;
			end = items + length;
		}
	}

	void Clear()
	{
		SetLength(0);
	}

	void Assign(const InstanceType &v)
	{
		int l = v.length;
		T * cur = items + l;
		for(; cur < end; cur++) cur->~T();
		SetCapacity(l);
		cur = items + l;
		T * i = items, * j = v.items;
		if(end > cur) end = cur;
		for(; i < end; i++, j++) *i = *j;
		for(; i < cur; i++, j++) new(i) T(*j);
		length = l;
		end = cur;
	}

	void Assign(int l, const T &item)
	{
		T * cur = items + l;
		for(; cur < end; cur++) cur->~T();
		Capacity = l;
		cur = items + l;
		T * i = items;
		if(end > cur) end = cur;
		for(; i < end; i++) *i = item;
		for(; i < cur; i++) new(i) T(item);
		length = l;
		end = cur;
	}

	void Fill(const T &item, int o = 0, int l = -1)
	{
		T * s = items + o, * e = s + (l < 0 ? length : l);
		if(e > end) e = end;
		for(T * i = s; i < e; i++) *i = item;
	}

	InstanceType & operator = (const InstanceType &v)
	{
		Assign(v);
		return *this;
	}

	T * Data()
	{
		return items;
	}

	const T * Data() const
	{
		return items;
	}

	operator T * ()
	{
		return items;
	}

	operator const T * () const
	{
		return items;
	}

	T * Begin()
	{
		return items;
	}

	const T * Begin() const
	{
		return items;
	}

	T * End()
	{
		return end;
	}

	const T * End() const
	{
		return end;
	}

	T & operator[](int index)
	{
		CheckIndex(index);
		return items[index];
	}

	const T & operator[](int index) const
	{
		CheckIndex(index);
		return items[index];
	}

	T & First()
	{
		CheckIndex(0);
		return *items;
	}

	const T & First() const
	{
		CheckIndex(0);
		return *items;
	}

	T & Last()
	{
		CheckIndex(length - 1);
		return *(end - 1);
	}

	const T & Last() const
	{
		CheckIndex(length - 1);
		return *(end - 1);
	}

	void Copy(T * it, int o = 0, int l = -1) const
	{
		T * s = items + o, * e = s + (l < 0 ? length : l);
		if(e > end) e = end;
		for(T *i = s; i < e; i++, it++) *it = *i;
	}

	int Add(const T &item)
	{
		int l = length;
		SetCapacity(l + 1);
		new (end) T(item);
		length++;
		end++;
		return l;
	}

	void Add(const InstanceType &v)
	{
		SetCapacity(length + v.length);
		for(T * i = end, * j = v.items; j < v.end; i++, j++) new(i) T(*j); 
		length += v.length;
		end += v.length;
	}

	void Insert(int index, const T &item)
	{
		CheckIndex(index, true);
		SetCapacity(length + 1);
		T *cur = items + index;
		MoveItems(cur + 1, cur, length - index);
		new((void*)cur) T(item);
		length++;
		end++;
	}

	void AddFront(const T &item)
	{
		Insert(0, item);
	}

	void Remove(int index, int count = 1)
	{
		CheckIndex(index);
		T *cur = items + index, *cur_end = cur + count;
		for(T *i = cur; i < cur_end; i++) i->~T();
		MoveItems(cur, cur_end, length - index - count);
		SetCapacity(length - count);
		length -= count;
		end -= count;
	}

	void RemoveFirst(int count = 1)
	{
		Remove(0, count);
	}

	void RemoveLast(int count = 1)
	{
		Remove(length - count, count);
	}

	bool Contains(const T &item)
	{
		for(const T * i = items; i < end; i++)
			if(*i == item) return true;
		return false;
	}

	int IndexOf(const T &item)
	{
		int j = 0;
		for(const T * i = items; i < end; i++, j++)
			if(*i == item) return j;
		return -1;
	}
};
