import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * Settings - singleton class for VFinger Demo option storage
 *
 * @version 1.0 02/07/2002
 */
public class Settings {
        public static final int IMAGE_SOURCE_FILE = 0;
        public static final int IMAGE_SOURCE_SCANNER = 1;

        private static Properties props;

        public static String dbFile;

        public static int imageSource;
        public static int startupMode;


        // VFinger
        public static int vfMode;
        public static int imageDpi;
        public static int falseAcceptanceRate;
        public static int matchingSpeed;
        public static int maximalRotation;
        public static boolean useBlockedOrientation;
        public static boolean useLowQualityImages;

        // registration
        public static String regParam;
        public static String registrationKey;

        static {
                props = new Properties();
        }

        /**
         * Hidden constructor
         * */
        private Settings() {
        };

        public static void reset() {
                dbFile = "vfinger.db";
                imageSource = 0;
                startupMode = 0;
                vfMode = 0;
                imageDpi = 500;
                falseAcceptanceRate = 1;
                matchingSpeed = 1;
                maximalRotation = 180;
                useBlockedOrientation = false;
                useLowQualityImages = false;

                regParam = "";
                registrationKey = "0";
        }

        /**
         * Save properties to file
         * @param fileName file to use
         * */
        public static void save(String fileName) throws FileNotFoundException {
                props.setProperty("dbFile", dbFile);
                props.setProperty("imageSource", Integer.toString(imageSource));
                props.setProperty("startupMode", Integer.toString(startupMode));
                props.setProperty("vfMode", Integer.toString(vfMode));
                props.setProperty("imageDpi", Integer.toString(imageDpi));
                props.setProperty("falseAcceptanceRate", Integer.toString(falseAcceptanceRate));
                props.setProperty("matchingSpeed", Integer.toString(matchingSpeed));
                props.setProperty("maximalRotation", Integer.toString(maximalRotation));
                props.setProperty("useBlockedOrientation", Boolean.toString(useBlockedOrientation));
                props.setProperty("useLowQualityImages", Boolean.toString(useLowQualityImages));
                props.setProperty("regParam", regParam);
                props.setProperty("registrationKey", registrationKey);

                FileOutputStream out = new FileOutputStream(fileName);
                try {
                        props.store(out, "VFingerDemo");
                        out.close();
                }
                catch (Exception e) {
                        System.out.println("Error saving setttings - " + e.getMessage());
                        e.printStackTrace();
                }
        }

        /**
         * Load properties from file
         * @param fileName properties file
         * */
        public static void load(String fileName) throws FileNotFoundException {
                props.clear();

                FileInputStream in = new FileInputStream(fileName);
                try {
                        props.load(in);

                        dbFile = props.getProperty("dbFile", "vfinger.db");
                        imageSource = Integer.parseInt(props.getProperty("imageSource", "0"));
                        startupMode = Integer.parseInt(props.getProperty("startupMode", "0"));
                        vfMode = Integer.parseInt(props.getProperty("vfMode", "0"));
                        imageDpi = Integer.parseInt(props.getProperty("imageDpi", "500"));
                        falseAcceptanceRate = Integer.parseInt(props.getProperty("falseAcceptanceRate", "1"));
                        matchingSpeed = Integer.parseInt(props.getProperty("matchingSpeed", "1"));
                        maximalRotation = Integer.parseInt(props.getProperty("maximalRotation", "180"));
                        useBlockedOrientation = Boolean.valueOf(props.getProperty("useBlockedOrientation", "false")).booleanValue();
                        useLowQualityImages = Boolean.valueOf(props.getProperty("useLowQualityImages", "false")).booleanValue();
                        regParam = props.getProperty("regParam", "0");
                        registrationKey = props.getProperty("registrationKey", "0");
                        in.close();
                }
                catch (Exception e) {
                        reset();
                        System.out.println(e.getMessage());
                        e.printStackTrace();
                }
        }
}

