unit VFFeatures;

interface

uses SysUtils, Math, VFinger, VFImage;

type
  TVFMinutiae = class
  private
    FCount: Integer;
    FCapacity: Integer;
    FItems: array of TVFMinutia;

    procedure CheckIndex(Index: Integer);
    procedure SetCount(const Value: Integer);
    function GetItem(Index: Integer): TVFMinutia;
    procedure SetItem(Index: Integer; const Value: TVFMinutia);

  public
    constructor Create;
    destructor Destroy; override;

    procedure Assign(Value: TVFMinutiae);
    procedure Clear;
    function Add(Item: TVFMinutia): Integer;
    procedure Insert(Index: Integer; Item: TVFMinutia);
    procedure Delete(Index: Integer);

    property Count: Integer read FCount write SetCount;
    property Capacity: Integer read FCapacity;
    property Items[Index: Integer]: TVFMinutia read GetItem write SetItem; default;
  end;

  TVFSingularPoints = class
  private
    FCount: Integer;
    FCapacity: Integer;
    FItems: array of TVFSingularPoint;

    procedure CheckIndex(Index: Integer);
    procedure SetCount(const Value: Integer);
    function GetItem(Index: Integer): TVFSingularPoint;
    procedure SetItem(Index: Integer; const Value: TVFSingularPoint);

  public
    constructor Create;
    destructor Destroy; override;

    procedure Assign(Value: TVFSingularPoints);
    procedure Clear;
    function Add(Item: TVFSingularPoint): Integer;
    procedure Insert(Index: Integer; Item: TVFSingularPoint);
    procedure Delete(Index: Integer);

    property Count: Integer read FCount write SetCount;
    property Capacity: Integer read FCapacity;
    property Items[Index: Integer]: TVFSingularPoint read GetItem write SetItem; default;
  end;

  TVFFeatures = class
  private
    FData: PByte;
    FDataLength: LongWord;
    FG: Byte;
    FMinutiae: TVFMinutiae;
    FSingularPoints: TVFSingularPoints;
    FBlockedOrientations: TVFImage;

  public
    constructor Create;
    constructor CreateFromData(Data: PByte; DataLength: LongWord);
    destructor Destroy; override;

    procedure Assign(Value: TVFFeatures);
    procedure Clear;
    procedure SetData(Data: PByte; DataLength: LongWord);
    function GetData(Data: PByte): LongWord;

    property G: Byte read FG write FG;
    property Minutiae: TVFMinutiae read FMinutiae;
    property SingularPoints: TVFSingularPoints read FSingularPoints;
    property BlockedOrientations: TVFImage read FBlockedOrientations;
  end;

implementation

{ TVFMinutiae }

function TVFMinutiae.Add(Item: TVFMinutia): Integer;
begin
  if FCount = FCapacity then
  begin
    Inc(FCapacity, 10);
    SetLength(FItems, FCapacity);
  end;
  FItems[FCount] := Item;
  Result := FCount;
  Inc(FCount);
end;

procedure TVFMinutiae.Assign(Value: TVFMinutiae);
begin
  if Value = Self then Exit;
  Count := Value.FCount;
  if FCount <> 0 then Move(Value.FItems[0], FItems[0], FCount * SizeOf(TVFMinutia));
end;

procedure TVFMinutiae.CheckIndex(Index: Integer);
begin
  if (Index < 0) or (Index >= FCount) then
    raise Exception.Create('Minutia index out of bounds');
end;

procedure TVFMinutiae.Clear;
begin
  FItems := nil;
  FCount := 0;
  FCapacity := 0;
  inherited;
end;

constructor TVFMinutiae.Create;
begin

end;

procedure TVFMinutiae.Delete(Index: Integer);
begin
  CheckIndex(Index);
  Dec(FCount);
  Move(FItems[Index + 1], FItems[Index], (FCount - Index) * SizeOf(TVFMinutia));
  if FCapacity - 10 >= FCount then
  begin
    Dec(FCapacity, 10);
    SetLength(FItems, FCapacity);
  end;
end;

destructor TVFMinutiae.Destroy;
begin
  Clear;
  inherited;
end;

function TVFMinutiae.GetItem(Index: Integer): TVFMinutia;
begin
  CheckIndex(Index);
  Result := FItems[Index];
end;

procedure TVFMinutiae.Insert(Index: Integer; Item: TVFMinutia);
begin
  CheckIndex(Index);
  if FCount = FCapacity then
  begin
    Inc(FCapacity, 10);
    SetLength(FItems, FCapacity);
  end;
  Move(FItems[Index], FItems[Index + 1], (FCount - Index) * SizeOf(TVFMinutia));
  Inc(FCount);
  FItems[Index] := Item;
end;

procedure TVFMinutiae.SetCount(const Value: Integer);
begin
  if FCount <> Value then
  begin
    FCount := Value;
    if FCount > FCapacity then
    begin
      FCapacity := FCount div 10;
      if FCount mod 10 <> 0 then Inc(FCapacity);
      FCapacity := FCapacity * 10;
      SetLength(FItems, FCapacity);
    end;
  end;
end;

procedure TVFMinutiae.SetItem(Index: Integer; const Value: TVFMinutia);
begin
  CheckIndex(Index);
  FItems[Index] := Value;
end;

{ TVFSingularPoints }

function TVFSingularPoints.Add(Item: TVFSingularPoint): Integer;
begin
  if FCount = FCapacity then
  begin
    Inc(FCapacity, 10);
    SetLength(FItems, FCapacity);
  end;
  FItems[FCount] := Item;
  Result := FCount;
  Inc(FCount);
end;

procedure TVFSingularPoints.Assign(Value: TVFSingularPoints);
begin
  if Value = Self then Exit;
  Count := Value.FCount;
  if FCount <> 0 then Move(Value.FItems[0], FItems[0], FCount * SizeOf(TVFSingularPoint));
end;

procedure TVFSingularPoints.CheckIndex(Index: Integer);
begin
  if (Index < 0) or (Index >= FCount) then
    raise Exception.Create('Singular point index out of bounds');
end;

procedure TVFSingularPoints.Clear;
begin
  FItems := nil;
  FCount := 0;
  FCapacity := 0;
end;

constructor TVFSingularPoints.Create;
begin

end;

procedure TVFSingularPoints.Delete(Index: Integer);
begin
  CheckIndex(Index);
  Dec(FCount);
  Move(FItems[Index + 1], FItems[Index], (FCount - Index) * SizeOf(TVFSingularPoint));
  if FCapacity - 10 >= FCount then
  begin
    Dec(FCapacity, 10);
    SetLength(FItems, FCapacity);
  end;
end;

destructor TVFSingularPoints.Destroy;
begin
  Clear;
  inherited;
end;

function TVFSingularPoints.GetItem(Index: Integer): TVFSingularPoint;
begin
  CheckIndex(Index);
  Result := FItems[Index];
end;

procedure TVFSingularPoints.Insert(Index: Integer; Item: TVFSingularPoint);
begin
  CheckIndex(Index);
  if FCount = FCapacity then
  begin
    Inc(FCapacity, 10);
    SetLength(FItems, FCapacity);
  end;
  Move(FItems[Index], FItems[Index + 1], (FCount - Index) * SizeOf(TVFSingularPoint));
  Inc(FCount);
  FItems[Index] := Item;
end;

procedure TVFSingularPoints.SetCount(const Value: Integer);
begin
  if FCount <> Value then
  begin
    FCount := Value;
    if FCount > FCapacity then
    begin
      FCapacity := FCount div 10;
      if FCount mod 10 <> 0 then Inc(FCapacity);
      FCapacity := FCapacity * 10;
      SetLength(FItems, FCapacity);
    end;
  end;
end;

procedure TVFSingularPoints.SetItem(Index: Integer; const Value: TVFSingularPoint);
begin
  CheckIndex(Index);
  FItems[Index] := Value;
end;

{ TVFFeatures }

procedure TVFFeatures.Assign(Value: TVFFeatures);
begin
  if Value = Self then Exit;
  SetData(Value.FData, Value.FDataLength);
end;

procedure TVFFeatures.Clear;
begin
  FreeMem(FData); FData := nil;
  FDataLength := 0;
  FG := 255;
  FMinutiae.Clear;
  FSingularPoints.Clear;
  FBlockedOrientations.Clear;
end;

constructor TVFFeatures.Create;
begin
  FData := nil;
  FDataLength := 0;
  FG := 255;
  FMinutiae := TVFMinutiae.Create;
  FSingularPoints := TVFSingularPoints.Create;
  FBlockedOrientations := TVFImage.Create;
end;

constructor TVFFeatures.CreateFromData(Data: PByte; DataLength: LongWord);
begin
  FData := nil;
  FDataLength := 0;
  FMinutiae := TVFMinutiae.Create;
  FSingularPoints := TVFSingularPoints.Create;
  FBlockedOrientations := TVFImage.Create;
  SetData(Data, DataLength);
end;

destructor TVFFeatures.Destroy;
begin
  Clear;
  FMinutiae.Free;
  FSingularPoints.Free;
  FBlockedOrientations.Free;
  inherited;
end;

function TVFFeatures.GetData(Data: PByte): LongWord;
begin
  Move(FData^, Data^, FDataLength);
  Result := FDataLength;
end;

procedure TVFFeatures.SetData(Data: PByte; DataLength: LongWord);
var
  Res: Integer;
  BOWidth, BOHeight: Integer;
begin
  Clear;
  FData := AllocMem(DataLength);
  Move(Data^, FData^, DataLength);
  FDataLength := DataLength;

  Res := VFFeatGetG(Data); VFCheckResult(Res);
  FG := Byte(Res);

  Res := VFFeatGetMinutiaCount(Data); VFCheckResult(Res);
  FMinutiae.Count := Res;
  Res := VFFeatGetMinutiae(Data, @FMinutiae.FItems[0]); VFCheckResult(Res);

  Res := VFFeatGetSPCount(Data); VFCheckResult(Res);
  FSingularPoints.Count := Res;
  Res := VFFeatGetSP(Data, @FSingularPoints.FItems[0]); VFCheckResult(Res);

  Res := VFFeatGetBOSize(Data, BOWidth, BOHeight); VFCheckResult(Res);
  FBlockedOrientations.SetSize(BOWidth, BOHeight);
  Res := VFFeatGetBO(Data, FBlockedOrientations.Bits); VFCheckResult(Res);
end;

end.

