/********************************************************\
|*                                                      *|
|*                  VeriFinger 5.0 SDK                  *|
|*                                                      *|
|*  VFinger.h                                           *|
|*    Header file for VeriFinger library                *|
|*                                                      *|
|*  Copyright (C) 1998-2006 Neurotechnologija           *|
|*                                                      *|
\********************************************************/


// Avoid multiple includes
#ifndef __VFINGER_INCLUDED__
#define __VFINGER_INCLUDED__


// Force C (not C++) names for VeriFinger API functions
#ifdef __cplusplus
extern "C"
{
#endif


///////////
//       //
// Types //
//       //
///////////

typedef unsigned char BYTE;
typedef signed char SBYTE;
typedef char CHAR;
typedef unsigned short WORD;
typedef unsigned short USHORT;
typedef short SHORT;
typedef unsigned long DWORD;
typedef unsigned int UINT;
typedef unsigned long ULONG;
typedef int INT;
typedef long LONG;
typedef float FLOAT;
typedef double DOUBLE;
typedef int BOOL;

typedef void * HVFCONTEXT;


#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef VOID
#define VOID void
#endif

#ifndef NULL
#ifdef __cplusplus
#define NULL 0
#else
#define NULL ((void*)0)
#endif
#endif


////////////
//        //
// Consts //
//        //
////////////

#define VF_IMAGE_RESOLUTION             500
#define VF_MIN_IMAGE_RESOLUTION 50
#define VF_MAX_IMAGE_RESOLUTION 5000

#define VF_FEATURES_RESOLUTION 500

#define VF_MIN_IMAGE_DIMENSION  16
#define VF_MAX_IMAGE_DIMENSION  2048

#define VF_MAX_FEATURES_DIMENSION 2048

#define VF_MAX_MINUTIA_COUNT 1024

#define VF_MAX_SINGULAR_POINT_COUNT 64

#define VF_BLOCK_SIZE 16

#define VF_MAX_BLOCKED_ORIENTS_DIMENSION (VF_MAX_IMAGE_DIMENSION / VF_BLOCK_SIZE)

#define VF_MAX_FEATURES_SIZE 10000

#define VF_GENERALIZE_COUNT 3

// Directions
#define VFDIR_0                         0
#define VFDIR_45                        30
#define VFDIR_90                        (VFDIR_45 * 2)
#define VFDIR_135                       (VFDIR_45 * 3)
#define VFDIR_180                       (VFDIR_45 * 4)
#define VFDIR_225                       (VFDIR_45 * 5)
#define VFDIR_270                       (VFDIR_45 * 6)
#define VFDIR_315                       (VFDIR_45 * 7)
#define VFDIR_360                       (VFDIR_45 * 8)
#define VFDIR_UNKNOWN           127
#define VFDIR_BACKGROUND        255
// Conversion from VFDIR_XXX to degrees and vice a versa
#define VFDirToDeg(dir) ((dir) * 180 / VFDIR_180)
#define VFDegToDir(deg) ((deg) * VFDIR_180 / 180)

// Registration types
#define VF_RT_NOT_PROTECTED     0
#define VF_RT_UNREGISTERED      6
#define VF_RT_HASP                      1
#define VF_RT_PC                        2
#define VF_RT_UAREU                     4
#define VF_RT_LAN                       8

// Parameter types
#define VF_TYPE_VOID    0
#define VF_TYPE_BYTE    1
#define VF_TYPE_SBYTE   2
#define VF_TYPE_WORD    3
#define VF_TYPE_SHORT   4
#define VF_TYPE_DWORD   5
#define VF_TYPE_INT             6
#define VF_TYPE_BOOL    10
#define VF_TYPE_CHAR    20
#define VF_TYPE_STRING  100

// FAR and matching threshold ratio
#define VF_FARMT_COUNT 4
const DOUBLE VF_FARMT_FAR[VF_FARMT_COUNT] = {0.0, 1.0, 2.0, 3.0}; // FAR 10^-X %
const INT VF_FARMT_MT[VF_FARMT_COUNT] = {22, 34, 47, 63}; // Matching threshold


////////////////
//            //
// Structures //
//            //
////////////////

typedef struct _VFMATCHDETAILS
{
        DWORD size;
        INT similarity;
        INT rotation;
        INT trans_x;
        INT trans_y;
} VFMATCHDETAILS;

typedef struct _VFMATCHDETAILSEX
{
        DWORD size;
        INT similarity;
        INT rotation;
        INT trans_x;
        INT trans_y;
        INT mm_count;
        SHORT mm[2][VF_MAX_MINUTIA_COUNT];
} VFMATCHDETAILSEX;


///////////////
//           //
// Functions //
//           //
///////////////

#if defined(linux) || defined (__LINUX__)
	#define VFINGER_API
#else
	#define VFINGER_API __stdcall
#endif


// Registration
INT VFINGER_API VFRegistrationType();
INT VFINGER_API VFGenerateId(CHAR * serial, CHAR * id);
INT VFINGER_API VFRegister(CHAR * serial, CHAR * key);

// Initialization
INT VFINGER_API VFInitialize();
INT VFINGER_API VFFinalize();

// Contexts
HVFCONTEXT VFINGER_API VFCreateContext();
INT VFINGER_API VFFreeContext(HVFCONTEXT context);

// Parameters
INT VFINGER_API VFGetParameter(INT parameter, VOID * value, HVFCONTEXT context);
INT VFINGER_API VFSetParameter(INT parameter, DWORD value, HVFCONTEXT context);

// Features extraction
INT VFINGER_API VFExtract(INT width, INT height, BYTE * image, INT resolution, BYTE * features, DWORD * size, HVFCONTEXT context);

// Features generalization
INT VFINGER_API VFGeneralize(INT count, const BYTE * const * gen_features, BYTE * features, DWORD * size, HVFCONTEXT context);

// Verification
INT VFINGER_API VFVerify(const BYTE * features1, const BYTE * features2, VFMATCHDETAILS * md, HVFCONTEXT context);

// Identification
INT VFINGER_API VFIdentifyStart(const BYTE * test_features, HVFCONTEXT context);
INT VFINGER_API VFIdentifyNext(const BYTE * sample_features, VFMATCHDETAILS * md, HVFCONTEXT context);
INT VFINGER_API VFIdentifyEnd(HVFCONTEXT context);


/////////////////
//             //
// Error codes //
//             //
/////////////////

// Pass VFXxx() function result to this macro to determine if the execution of that function failed
#define VFFailed(result) ((result) < 0)

// Pass VFXxx() function result to this macro to determine if the execution of that function succeeded
#define VFSucceeded(result) ((result) >= 0)

// General
#define VFE_OK                                  0
#define VFE_FAILED                              -1
#define VFE_OUT_OF_MEMORY               -2
#define VFE_NOT_INITIALIZED             -3
#define VFE_ARGUMENT_NULL               -4
#define VFE_INVALID_ARGUMENT    -5
// Registration
#define VFE_NOT_REGISTERED                              -2000
#define VFE_INVALID_SERIAL_NUMBER               -2001
#define VFE_INVALID_REGISTRATION_KEY    -2002
#define VFE_SCANNER_DRIVER_ERROR                -2003
#define VFE_REGISTRATION_NOT_NEEDED             -2004
#define VFE_NO_SCANNER                                  -2005
#define VFE_MORE_THAN_ONE_SCANNER               -2006
#define VFE_LM_CONNECTION_ERROR                 -2007
#define VFE_LM_NO_MORE_LICENSES                 -2008
// Parameters
#define VFE_INVALID_PARAMETER   -10
#define VFE_PARAMETER_READ_ONLY -11
// Features extraction
#define VFE_ILLEGAL_IMAGE_RESOLUTION    -101
#define VFE_ILLEGAL_IMAGE_SIZE                  -102
#define VFE_LOW_QUALITY_IMAGE                   -103
// VeriFinger specific
#define VFE_INVALID_MODE        -1000


////////////////
//            //
// Parameters //
//            //
////////////////

/**************************************************************************************\

Parameters can be of the following types:
        byte (VF_TYPE_BYTE)
        signed byte (VF_TYPE_SBYTE)
        word (VF_TYPE_WORD)
        short integer (VF_TYPE_SHORT)
        double word (VF_TYPE_DWORD)
        integer (VF_TYPE_INT)
        boolean (VF_TYPE_BOOL)
        char (VF_TYPE_CHAR);
        string (VF_TYPE_STRING)
For VFGetParameters "VOID * value" have to be:
        BYTE* for byte
        SBYTE* for signed byte
        WORD* for word
        SHORT* for short integer
        DWORD* for double word
        INT* for integer
        BOOL* for boolean
        CHAR* for char
        CHAR* for string (must point to array of at least string length + 1 CHARs; if NULL string length is returned)
For VFSetParameters "DWORD value" have to be:
        (DWORD)(BYTE) for byte;
        (DWORD)(SBYTE) for signed byte
        (DWORD)(WORD) for word
        (DWORD)(SHORT) for short integer
        (DWORD) for double word
        (DWORD)(INT) for integer
        (DWORD)(BOOL) for boolean
        (DWORD)(CHAR) for char
        (DWORD)(CHAR*) for string
Some parameters are read only

\**************************************************************************************/

// General
#define VFP_TYPE                        0 // Get | Pass parameter identifier ((VOID*)VFP_XXX) as value | Returns one of VF_TYPE_XXX constants
#define VFP_NAME                        10 // Get | String
#define VFP_VERSION_HIGH        11 // Get | DWord
#define VFP_VERSION_LOW         12 // Get | DWord
#define VFP_COPYRIGHT           13 // Get | String

// Features extraction
#define VFP_EXTRACT_FEATURES                                                            110 // Get/Set | Int | Can be one of the following
        #define VF_ALL_FEATURES                                                                 0
        #define VF_OMIT_BLOCKED_ORIENTATIONS                                    1
        #define VF_OMIT_BLOCKED_ORIENTATIONS_AND_CURVATURES             2

// Features Matching
#define VFP_MATCHING_THRESHOLD                  200 // Get/Set | Int | >= 0
#define VFP_MAXIMAL_ROTATION                    201 // Get/Set | Int | DIR_0..DIR_180
#define VFP_MATCH_FEATURES                              210 // Get/Set | Int | The values are the same as for the VFP_EXTRACT_FEATURES
#define VFP_MATCHING_SPEED                              220 // Get/Set | Int | Can be one of the following
        #define VF_MATCHING_SPEED_LOW           0
        #define VF_MATCHING_SPEED_HIGH          256

// Features generalization
#define VFP_GENERALIZATION_THRESHOLD    300 // Get/Set | Int | >= 0

// VeriFinger specific
#define VFP_MODE                                                                        1000 // Get/Set | Int | Can be one of the following
        #define VF_MODE_GENERAL                                                 0
        #define VF_MODE_DIGITALPERSONA_URU                              100
        #define VF_MODE_BIOMETRIKA_FX2000                               200
        #define VF_MODE_KEYTRONIC_SECUREDESKTOP                 300
        #define VF_MODE_IDENTIX_TOUCHVIEW                               400
        #define VF_MODE_PRECISEBIOMETRICS_100CS                 500
        #define VF_MODE_STMICROELECTRONICS_TOUCHCHIP    600
        #define VF_MODE_IDENTICATORTECHNOLOGY_DF90              700
        #define VF_MODE_AUTHENTEC_AFS2                                  800
        #define VF_MODE_AUTHENTEC_AES4000                               810
        #define VF_MODE_ATMEL_FINGERCHIP                                900
        #define VF_MODE_BMF_BLP100                                              1000
        #define VF_MODE_SECUGEN_HAMSTER                                 1100
        #define VF_MODE_ETHENTICA                                               1200


//////////////
//          //
// Features //
//          //
//////////////

/**************************************************************************************************\

// Features structure
{
        BYTE Format; // Defines flags for identifying features format. See Features format section
        BYTE G; // Global feature, average ridge density, used for preclassification
        BYTE Offsets[3]; // Offsets of minutiae and singular points coordinates (x and y)
        USHORT MinutiaCount; // Count of minutia in the following array
        {
                DWORD Minutia; // Minutia data
                [BYTE MinutiaC;] // Minutia curvature, optional.
                                 // If present VFFEAT_HAS_CURVATURES flag must be set in the Format field
        } [MinutiaCount]; // Array of minutia data
        [{
                BYTE SPCount; // Count of singular points in SP array
                DWORD SP[SPCount]; // Array of singular point data
        }] // Singular points data, optional.
           // If present VFFEAT_HAS_SINGULAR_POINTS flag must be set in the Format field
        [{
                BYTE BOWidth; // Width of blocked orientations image minus one
                BYTE BOHeight; // Height of blocked orientations image minus one
                BYTE BOBits[]; // Compressed data of blocked orientations image
        }] // Blocked orientations data, optional.
           // If present VFFEAT_BLOCKED_ORIENTATIONS flag must be set in the Format field
}

// BYTE Offsets[3] format is: xxxx xxxx xxxx yyyy yyyy yyyy
// xxxx xxxx xxxx - offset of x coordinates (ox) + VF_MAX_FEATURES_DIMENSION (12 bits)
// yyyy yyyy yyyy - offset of y coordinates (oy) + VF_MAX_FEATURES_DIMENSION (12 bits)

// DWORD Minutia format is: xxxx xxxx xxxy yyyy yyyy yy-- dddd dddd
// DWORD SP format is:      xxxx xxxx xxxy yyyy yyyy yytt dddd dddd
// xxxx xxxx xxx - x coordinate of minutia or singular point (x) - ox (11 bits)
// y yyyy yyyy yy - y coordinate of minutia or singular point (y) - oy (11 bits)
// tt - type of singular point (t) (2 bits), see Singular point types section
// dddd dddd - direction of minutia or singular point (d) (8 bits)

// BYTE BOBits[]: each byte has a most significant bit as a marker.
// If not set then all other seven bits are orientation (DIR_0 .. DIR_180 - 1),
// if they are equal to VFFEAT_BO_BACKGROUND then orientation is DIR_BACKGROUND,
// in other cases orientation is DIR_UNKNOWN
// If most significant bit is set then byte should be interpreted as cccc nnnn
// cccc - code (4 bits)
// nnnn - number (4 bits)
// Code can be one of the following:
// VFFEAT_BO_EQUALS - next number + 1 pixels are the of the same orientation as previous
// VFFEAT_BO_DIFFS - next number + 1 bytes are of the following format:
        // iiii iiii iiii iiii .... ....
        // iiii - next pixel orientation difference with previous + 8 (4 bits)
// VFFEAT_BO_OFFx - if number is 0 then cccc nnnn evaluates to VFFEAT_BO_EOL - end of line
        // (all other pixels to the end of the line are DIR_BACKGROUND)
        // in other case skip number + 1 pixels (they are DIR_BACKGROUND)
// VFFEAT_BO_OFFy - if number is 0 then cccc nnnn evaluates to VFFEAT_BO_EOI - end of image
        // (all other pixels to the end of the image are DIR_BACKGOUND)
        // in other case skip number + 1 lines
        // (all pixels to the end of this line and next number + 1 lines are DIR_BACKGROUND)

\**************************************************************************************************/

// Features format
#define VFFEAT_HAS_CURVATURES                   0x01 // Set if features contain information about minutia curvature (MinutiaC member)
#define VFFEAT_HAS_SINGULAR_POINTS              0x02 // Set if features contain singular points (SPxxx members)
#define VFFEAT_HAS_BLOCKED_ORIENTATIONS 0x04 // Set if features contain blocked orientations (BOxxx members)

#define VFFeatFlagSet(flags, flag) ((BOOL)(((BYTE)(flags) & (BYTE)(flag)) == (BYTE)(flag)))

#define VFFeatFormat_(features) ((BYTE*)features)
#define VFFeatFormat(features) (*VFFeatFormat_(features))

#define VFFeatFormatFlagSet(features, flag) VFFeatFlagSet(VFFeatFormat(features), flag)

#define VFFeatHasCurvatures(features) VFFeatFormatFlagSet(features, VFFEAT_HAS_CURVATURES)
#define VFFeatHasSingularPoints(features) VFFeatFormatFlagSet(features, VFFEAT_HAS_SINGULAR_POINTS)
#define VFFeatHasBlockedOrientations(features) VFFeatFormatFlagSet(features, VFFEAT_HAS_BLOCKED_ORIENTATIONS)

// G
#define VFFeatG_(features) (BYTE*)(VFFeatFormat_(features) + 1)
#define VFFeatG(features) (*VFFeatG_(features))

// Offsets
#define VFFEAT_OX_SHIFT 12
#define VFFEAT_OY_SHIFT 0
#define VFFEAT_OFFSETS_LENGTH 3

#define VFFEAT_OX_MASK 0x00FFF000
#define VFFEAT_OY_MASK 0x00000FFF

#define VFFeatOX(data) ((SHORT)((SHORT)(((DWORD)(data) & VFFEAT_OX_MASK) >> VFFEAT_OX_SHIFT) - VF_MAX_FEATURES_DIMENSION))
#define VFFeatOY(data) ((SHORT)((SHORT)(((DWORD)(data) & VFFEAT_OY_MASK) >> VFFEAT_OY_SHIFT) - VF_MAX_FEATURES_DIMENSION))

#define VFFeatOffsets_(features) ((DWORD*)(VFFeatG_(features) + 1))
#define VFFeatOffsets(features) (*VFFeatOffsets_(features))
#define VFFeatOffsetX(features) VFFeatOX(VFFeatOffsets(features))
#define VFFeatOffsetY(features) VFFeatOY(VFFeatOffsets(features))

// Minutiae and singular points
#define VFFeatMinutiaCount_(features) ((SHORT*)((BYTE*)VFFeatOffsets_(features) + VFFEAT_OFFSETS_LENGTH))
#define VFFeatMinutiaCount(features) (*VFFeatMinutiaCount_(features))
#define VFFeatMinutia_(features, index) ((DWORD*)((BYTE*)(VFFeatMinutiaCount_(features) + 1) + (INT)index * (sizeof(DWORD) + (VFFeatHasCurvatures(features) ? 1 : 0))))
#define VFFeatMinutia(features, index) (*VFFeatMinutia_(features, index))
#define VFFeatSPCount_(features) ((SBYTE*)VFFeatMinutia_(features, VFFeatMinutiaCount(features)))
#define VFFeatSPCount(features) (*VFFeatSPCount_(features))
#define VFFeatSP_(features, index) ((DWORD*)(VFFeatSPCount_(features) + 1) + (INT)index)
#define VFFeatSP(features, index) (*VFFeatSP_(features, index))

#define VFFEAT_X_SHIFT  21
#define VFFEAT_Y_SHIFT  10
#define VFFEAT_T_SHIFT  8

#define VFFEAT_X_MASK   0xFFE00000
#define VFFEAT_Y_MASK   0x001FFC00
#define VFFEAT_T_MASK   0x00000300
#define VFFEAT_D_MASK   0x000000FF

#define VFFeatX(data, ox) ((SHORT)((((DWORD)(data) & VFFEAT_X_MASK) >> VFFEAT_X_SHIFT) + (SHORT)ox))
#define VFFeatY(data, oy) ((SHORT)((((DWORD)(data) & VFFEAT_Y_MASK) >> VFFEAT_Y_SHIFT) + (SHORT)oy))
#define VFFeatT(data) ((BYTE)(((DWORD)(data) & VFFEAT_T_MASK) >> VFFEAT_T_SHIFT))
#define VFFeatD(data) ((BYTE)((DWORD)(data) & VFFEAT_D_MASK))

#define VFFeatMinutiaX(features, index) VFFeatX(VFFeatMinutia(features, index), VFFeatOffsetX(features))
#define VFFeatMinutiaY(features, index) VFFeatY(VFFeatMinutia(features, index), VFFeatOffsetY(features))
#define VFFeatMinutiaD(features, index) VFFeatD(VFFeatMinutia(features, index))
#define VFFeatMinutiaC_(features, index) ((BYTE*)(VFFeatMinutia_(features, index) + 1))
#define VFFeatMinutiaC(features, index) (*VFFeatMinutiaC_(features, index))

#define VFFeatSPX(features, index) VFFeatX(VFFeatSP(features, index), VFFeatOffsetX(features))
#define VFFeatSPY(features, index) VFFeatY(VFFeatSP(features, index), VFFeatOffsetY(features))
#define VFFeatSPT(features, index) VFFeatT(VFFeatSP(features, index))
#define VFFeatSPD(features, index) VFFeatD(VFFeatSP(features, index))

// Singular point types
#define VFFEAT_SP_DELTA                 0x03
#define VFFEAT_SP_CORE                  0x01
#define VFFEAT_SP_DOUBLECORE    0x02

// Blocked orientations
#define VFFeatBOWidth_(features) ((BYTE*)VFFeatSP_(features, VFFeatSPCount(features)))
#define VFFeatBOWidth(features) ((SHORT)(*VFFeatBOWidth_(features) + 1))
#define VFFeatBOHeight_(features) ((BYTE*)(VFFeatBOWidth_(features) + 1))
#define VFFeatBOHeight(features) ((SHORT)(*VFFeatBOHeight_(features) + 1))
#define VFFeatBOBits(features) ((BYTE*)(VFFeatBOHeight_(features) + 1))

#define VFFEAT_BO_BACKGROUND 126
#define VFFEAT_BO_EQUALS        0x80
#define VFFEAT_BO_DIFFS         0xA0
#define VFFEAT_BO_OFFx          0xC0
#define VFFEAT_BO_OFFy          0xE0
#define VFFEAT_BO_EOL           VFFEAT_BO_OFFx
#define VFFEAT_BO_EOI           VFFEAT_BO_OFFy

#define VFFeatBOToDir(d) ((BYTE)((BYTE)(d) == VFFEAT_BO_BACKGROUND ? (BYTE)VFDIR_BACKGROUND : (d)))
#define VFFeatBOIsValue(d) ((BOOL)(((BYTE)(d) & 0x80) == 0))
#define VFFEAT_BO_CODE_MASK 0xE0
#define VFFeatBOGetCode(d) ((BYTE)((BYTE)d & VFFEAT_BO_CODE_MASK))
#define VFFEAT_BO_N_MASK 0x1F
#define VFFeatBOGetN(d) ((SBYTE)((BYTE)d & VFFEAT_BO_N_MASK))

#define VFFeatBOGetDiff1(d) ((SBYTE)((((BYTE)d & 0xF0) >> 4) - 8))
#define VFFeatBOGetDiff2(d) ((SBYTE)(((BYTE)d & 0x0F) - 8))

#define VFFeatBOMakeCode(code, n) ((BYTE)((code) | (n)))
#define VFFeatBOMakeEquals(n) VFFeatBOMakeCode(VFFEAT_BO_EQUALS, n)
#define VFFeatBOMakeDiffs(n) VFFeatBOMakeCode(VFFEAT_BO_DIFFS, n)
#define VFFeatBOMakeOffX(n) VFFeatBOMakeCode(VFFEAT_BO_OFFx, n)
#define VFFeatBOMakeOffY(n) VFFeatBOMakeCode(VFFEAT_BO_OFFy, n)


////////////////////////
//                    //
// Features functions //
//                    //
////////////////////////

///////////////
//           //
// Constants //
//           //
///////////////

#define VF_IMAGE_RESOLUTION 500
#define VF_FEATURES_RESOLUTION 500
#define VF_BLOCK_SIZE 16
#define VF_MAX_FEATURES_SIZE 10000

#define VF_GENERALIZE_COUNT 3

#ifndef PI
#define PI 3.1415926535897932384626433832795
#endif

//////////////
//          //
// Features //
//          //
//////////////

typedef enum _VFSingularPointType
{
	vfsptUnknown = 0,
	vfsptCore = 1,
	vfsptDoubleCore = 2,
	vfsptDelta = 3
} VFSingularPointType;

typedef struct _VFSingularPoint
{
	INT X;
	INT Y;
	VFSingularPointType T;
	BYTE D;
} VFSingularPoint;

typedef enum _VFMinutiaType
{
	vfmtUnknown = 0,
	vfmtEnd = 1,
	vfmtBifurcation = 2
} VFMinutiaType;

typedef struct _VFMinutia
{
	INT X;
	INT Y;
	VFMinutiaType T;
	BYTE D;
	BYTE C;
	BYTE G;
} VFMinutia;

///////////////////
//               //
// Match details //
//               //
///////////////////

typedef struct _VFMatchDetails
{
	DWORD Size;

	INT Similarity;
	INT Rotation;
	INT TranslationX;
	INT TranslationY;
} VFMatchDetails;

#define VF_MAX_MATCHED_MINUTIA_COUNT 1024

typedef struct _VFMatchedMinutiae
{
	INT Count;
	SHORT Test[VF_MAX_MATCHED_MINUTIA_COUNT];
	SHORT Sample[VF_MAX_MATCHED_MINUTIA_COUNT];
} VFMatchedMinutiae;

typedef struct _VFMatchDetailsEx
{
	DWORD Size;

	INT Similarity;
	INT Rotation;
	INT TranslationX;
	INT TranslationY;

	VFMatchedMinutiae MM;
} VFMatchDetailsEx;


// Features compression
	INT VFINGER_API VFFeatSet(BYTE g, INT mCount, const VFMinutia * m,
	INT spCount, const VFSingularPoint * sp, INT boWidth, INT boHeight, const BYTE * bo,
	BYTE * features);

// Features decompression
INT VFINGER_API VFFeatGetG(const BYTE * features);
INT VFINGER_API VFFeatGetMinutiaCount(const BYTE * features);
INT VFINGER_API VFFeatGetMinutiae(const BYTE * features, VFMinutia * m);
INT VFINGER_API VFFeatGetSPCount(const BYTE * features);
INT VFINGER_API VFFeatGetSP(const BYTE * features, VFSingularPoint * sp);
INT VFINGER_API VFFeatGetBOSize(const BYTE * features, INT * pWidth, INT * pHeight);
INT VFINGER_API VFFeatGetBO(const BYTE * features, BYTE * bo);


#ifdef __cplusplus
}
#endif


#endif //!__VFINGER_INCLUDED__
