using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Neurotec.Biometrics.VeriFingerDemo.Properties;
using System.Globalization;

namespace Neurotec.Biometrics.VeriFingerDemo
{
	public partial class MatchingOptionsForm : Form
	{
		#region Private static readonly fields

		private static readonly KeyValuePair<uint, string>[] vfmModes =
			{
				new KeyValuePair<uint, string>(VFMatcher.ModeGeneral, "General"),
				new KeyValuePair<uint, string>(VFMatcher.ModeDigitalPersonaUareU, "DigitalPersona U.are.U"),
				new KeyValuePair<uint, string>(VFMatcher.ModeBiometrikaFX2000, "BiometriKa FX2000"),
				new KeyValuePair<uint, string>(VFMatcher.ModeBiometrikaFX3000, "BiometriKa FX3000"),
				new KeyValuePair<uint, string>(VFMatcher.ModeKeytronicSecureDesktop, "Keytronic SecureDesktop"),
				new KeyValuePair<uint, string>(VFMatcher.ModeIdentixTouchView, "Identix TouchView"),
				new KeyValuePair<uint, string>(VFMatcher.ModeIdentixDfr2090, "Identix DFR2090"),
				new KeyValuePair<uint, string>(VFMatcher.ModePreciseBiometrics100CS, "PreciseBiometrics 100CS"),
				new KeyValuePair<uint, string>(VFMatcher.ModeUpekTouchChip, "Upek TouchChip"),
				new KeyValuePair<uint, string>(VFMatcher.ModeIdenticatorTechnologyDF90, "IdenticatorTechnology DF90"),
				new KeyValuePair<uint, string>(VFMatcher.ModeAuthentecAFS2, "Authentec AF-S2"),
				new KeyValuePair<uint, string>(VFMatcher.ModeAuthentecAes4000, "Authentec AES4000"),
				new KeyValuePair<uint, string>(VFMatcher.ModeAuthentecAes2501B, "Authentec AES2501B"),
				new KeyValuePair<uint, string>(VFMatcher.ModeAtmelFingerchip, "Atmel FingerChip"),
				new KeyValuePair<uint, string>(VFMatcher.ModeBmfBlp100, "BMF BLP100"),
				new KeyValuePair<uint, string>(VFMatcher.ModeSecugenHamster, "Secugen Hamster"),
				new KeyValuePair<uint, string>(VFMatcher.ModeEthentica, "Ethentica"),
				new KeyValuePair<uint, string>(VFMatcher.ModeCrossmatchVerifier300, "CrossMatch Verifier 300"),
				new KeyValuePair<uint, string>(VFMatcher.ModeNitgenFingkeyHamster, "Nitgen Fingkey Hamster"),
				new KeyValuePair<uint, string>(VFMatcher.ModeTestechBioI, "Testech Bio-i"),
				new KeyValuePair<uint, string>(VFMatcher.ModeDigentIzzix, "Digent Izzix"),
				new KeyValuePair<uint, string>(VFMatcher.ModeStartekFM200, "Startek FM200"),
				new KeyValuePair<uint, string>(VFMatcher.ModeFujitsuMBF200, "Fujitsu MBF200"),
				new KeyValuePair<uint, string>(VFMatcher.ModeFutronicFS80, "Futronic FS80"),
				new KeyValuePair<uint, string>(VFMatcher.ModeLighTuningLttC500, "LighTuning LTT-C500"),
				new KeyValuePair<uint, string>(VFMatcher.ModeTacomaCmos, "Tacoma CMOS")
			};
		private static readonly Dictionary<uint, int> vfmModesToIndexes = new Dictionary<uint, int>();

		#endregion

		#region Static constructor

		static MatchingOptionsForm()
		{
			int i;
			i = 0;
			foreach (KeyValuePair<uint, string> vfmMode in vfmModes)
			{
				vfmModesToIndexes.Add(vfmMode.Key, i++);
			}
		}

		#endregion

		#region Public constructor

		public MatchingOptionsForm()
		{
			InitializeComponent();

			vfmModeComboBox.BeginUpdate();
			foreach (KeyValuePair<uint, string> vfmMode in vfmModes)
			{
				vfmModeComboBox.Items.Add(vfmMode.Value);
			}
			vfmModeComboBox.EndUpdate();

			vfmFarComboBox.BeginUpdate();
			vfmFarComboBox.Items.Add(0.001.ToString("P1"));
			vfmFarComboBox.Items.Add(0.0001.ToString("P2"));
			vfmFarComboBox.Items.Add(0.00001.ToString("P3"));
			vfmFarComboBox.EndUpdate();

			UpdateSettings();
		}

		#endregion

		#region Private methods

		private void UpdateSettings()
		{
			Settings settings = Settings.Default;
			gPercentNumericUpDown.Value = settings.FPGPercent;
			vfmModeComboBox.SelectedIndex = vfmModesToIndexes[Data.VFMatcher.Mode];
			vfmFarComboBox.Text = Data.MatchingThresholdToString(Data.VFMatcher.MatchingThreshold);
			vfmMaximalRotationNumericUpDown.Value = Data.MaximalRotationToDegrees(Data.VFMatcher.MaximalRotation);
		}

		#endregion

		private void MatchingOptionsForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (DialogResult == DialogResult.OK)
			{
				string error = null;
				try
				{
					int vfmMatchingThreshold = Data.MatchingThresholdFromString(vfmFarComboBox.Text);
				}
				catch
				{
					vfmFarComboBox.Select();
					error = "FAR is not valid";
				}
				if (error != null)
				{
					MessageBox.Show(error, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
					e.Cancel = true;
				}
			}
		}

		private void MatchingOptionsForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			Settings settings = Settings.Default;
			if (DialogResult == DialogResult.OK)
			{
				settings.FPGPercent = (int)gPercentNumericUpDown.Value;
				Data.VFMatcher.Mode = vfmModes[vfmModeComboBox.SelectedIndex].Key;
				Data.VFMatcher.MatchingThreshold = Data.MatchingThresholdFromString(vfmFarComboBox.Text);
				Data.VFMatcher.MaximalRotation = Data.MaximalRotationFromDegrees((int)vfmMaximalRotationNumericUpDown.Value);
			}
			else
			{
				settings.Reload();
				Data.VFMatcher.Reset();
				Data.UpdateVfm();
			}
			Data.UpdateVfmSettings();
			settings.Save();
		}

		private void defaultButton_Click(object sender, EventArgs e)
		{
			Data.ResetSetting("MatchAllDatabase");
			Data.ResetSetting("ShowAllMatchingResults");
			Data.ResetSetting("FPUseG");
			Data.ResetSetting("FPGPercent");
			Data.VFMatcher.Reset();
			UpdateSettings();
		}
	}
}
