using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Neurotec.Biometrics.Gui;
using Neurotec.Images;
using Neurotec.Biometrics.VeriFingerDemo.Properties;
using System.Diagnostics;
using System.IO;

namespace Neurotec.Biometrics.VeriFingerDemo
{
	public partial class NFRecordForm : Form
	{
		#region Private static readonly fields

		private static readonly NRgb resultImageMinColor = new NRgb(0, 230, 0);
		private static readonly NRgb resultImageMaxColor = new NRgb(255, 255, 255);

		#endregion

		#region Private fields

		private bool enrollMode = true;
		private bool showID = true;
		private FPScannerMan scannerMan;
		private NFView nfView1, nfView2;
		private NGrayscaleImage[] images = new NGrayscaleImage[0];
		private NGrayscaleImage[] resultImages = new NGrayscaleImage[0];
		private Bitmap[] bitmaps = new Bitmap[0];
		private Bitmap[] resultBitmaps = new Bitmap[0];
		private byte[][] templates = new byte[0][];
		private int count = 0;
		private int availableCount = 0;
		private NGrayscaleImage image;
		private NGrayscaleImage resultImage;
		private byte[] template;

		#endregion

		#region Public constructor

		public NFRecordForm()
		{
			InitializeComponent();

			Settings settings = Settings.Default;

			Size = settings.NFRecordFromSize;

			nfView1 = new NFView();
			nfView1.Dock = DockStyle.Fill;
			nfView1.BorderStyle = BorderStyle.Fixed3D;
			nfViewSplitContainer.Panel1.Controls.Add(nfView1);
			nfView2 = new NFView();
			nfView2.Dock = DockStyle.Fill;
			nfView2.BorderStyle = BorderStyle.Fixed3D;
			nfView2.ShownImage = ShownImage.Result;
			nfView2.AllowSelection = false;
			nfView2.AllowHover = false;
			nfView2.ShownImageChanged += new EventHandler(nfView2_ShownImageChanged);
			nfView2.ShowMinutiaeChanged += new EventHandler(nfView2_ShowMinutiaeChanged);
			nfViewSplitContainer.Panel2.Controls.Add(nfView2);
			nfView2.ShownImage = settings.FPShownImage;
			nfView2.ShowMinutiae = settings.FPShowMinutiae;
			UpdateShownImage();
			UpdateShowMinutiae();

			string openFilter, saveFilter;
			Data.GetImageFileFilters(out openFilter, out saveFilter);
			openFileDialog.Filter = openFilter;
			saveFileDialog.Filter = saveFilter;

			OnEnrollModeChanged();
			OnShowIDChanged();

			if (settings.UseFPScannerMan)
			{
				bool useSelectedScanners = settings.UseSelectedFPScanners;
				string[] selectedScanners = useSelectedScanners ?
					(settings.SelectedFPScanners == null ? null : settings.SelectedFPScanners.Split(Path.PathSeparator))
					: null;
				scannerMan = new FPScannerMan(this);
				scannerMan.ScannerAdded += new FPScannerManScannerEventHandler(scannerMan_ScannerAdded);
				scannerMan.ScannerRemoved += new FPScannerManScannerEventHandler(scannerMan_ScannerRemoved);
				foreach (FPScanner scanner in scannerMan.Scanners)
				{
					if (!useSelectedScanners || Array.IndexOf<string>(selectedScanners, scanner.ID) != -1)
					{
						OnNewScanner(scanner);
					}
				}
			}
		}

		#endregion

		#region Private methods

		private void UpdateShownImage()
		{
			shownImageNoneToolStripMenuItem.Checked = nfView2.ShownImage == ShownImage.None;
			shownImageOriginalToolStripMenuItem.Checked = nfView2.ShownImage == ShownImage.Original;
			shownImageResultToolStripMenuItem.Checked = nfView2.ShownImage == ShownImage.Result;
		}

		private void UpdateShowMinutiae()
		{
			showMinutiaeToolStripMenuItem.Checked = nfView2.ShowMinutiae;
		}

		void nfView2_ShownImageChanged(object sender, EventArgs e)
		{
			UpdateShownImage();
		}

		void nfView2_ShowMinutiaeChanged(object sender, EventArgs e)
		{
			UpdateShowMinutiae();
		}

		private void scannerMan_ScannerAdded(object sender, FPScannerManScannerEventArgs e)
		{
			LogLine("Scanner plugged: {0}", e.Scanner.ID);
			Settings settings = Settings.Default;
			bool useSelectedScanners = settings.UseSelectedFPScanners;
			string[] selectedScanners = useSelectedScanners ?
				(settings.SelectedFPScanners == null ? null : settings.SelectedFPScanners.Split(Path.PathSeparator))
				: null;
			if (!useSelectedScanners || Array.IndexOf<string>(selectedScanners, e.Scanner.ID) != -1)
			{
				OnNewScanner(e.Scanner);
			}
		}

		private void scannerMan_ScannerRemoved(object sender, FPScannerManScannerEventArgs e)
		{
			LogLine("Scanner unplugged: {0}", e.Scanner.ID);
		}

		private void OnNewScanner(FPScanner scanner)
		{
			scanner.IsCapturingChanged += new EventHandler(scanner_IsCapturingChanged);
			scanner.FingerPlaced += new EventHandler(scanner_FingerPlaced);
			scanner.FingerRemoved += new EventHandler(scanner_FingerRemoved);
			scanner.ImageScanned += new FPScannerImageScannedEventHandler(scanner_ImageScanned);
			try
			{
				scanner.StartCapturing();
			}
			catch (Exception e)
			{
				string text = string.Format("Error starting capturing on scanner {0}: {1}", scanner.ID, e.Message);
				LogLine(text);
				MessageBox.Show(text, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void scanner_IsCapturingChanged(object sender, EventArgs e)
		{
			FPScanner scanner = (FPScanner)sender;
			if (scanner.IsCapturing)
			{
				LogLine("Scanner {0} started capturing", scanner.ID);
			}
			else
			{
				LogLine("Scanner {0} stopped capturing", scanner.ID);
			}
		}

		private void scanner_FingerPlaced(object sender, EventArgs e)
		{
			FPScanner scanner = (FPScanner)sender;
			LogLine("Finger placed onto scanner {0}", scanner.ID);
		}

		private void scanner_FingerRemoved(object sender, EventArgs e)
		{
			FPScanner scanner = (FPScanner)sender;
			LogLine("Finger removed from scanner {0}", scanner.ID);
		}

		private void scanner_ImageScanned(object sender, FPScannerImageScannedEventArgs ea)
		{
			FPScanner scanner = (FPScanner)sender;
			OnImage((NGrayscaleImage)ea.Image.Clone(), string.Format("Image scanned from scanner {0}", scanner.ID));
		}

		private void OnImageChanged()
		{
			saveToolStripSplitButton.Enabled = saveImageToolStripMenuItem.Enabled = image != null;
		}

		private void OnResultImageChanged()
		{
			saveResultImageToolStripMenuItem.Enabled = resultImage != null;
		}

		private void ClearView()
		{
			nfView1.Image = null;
			if (nfView2.Template != null)
			{
				nfView2.Template.Dispose();
				nfView2.Template = null;
			}
			nfView2.Image = null;
			nfView2.ResultImage = null;
			image = null;
			resultImage = null;
			OnImageChanged();
			OnResultImageChanged();
		}

		private void Clear()
		{
			ClearView();
			Settings settings = Settings.Default;
			count = enrollMode && settings.FPUseGeneralization ? settings.FPGeneralizationTemplates : 1;
			foreach (NGrayscaleImage image in images)
			{
				if (image != null) image.Dispose();
			}
			foreach (NGrayscaleImage resultImage in resultImages)
			{
				if (resultImage != null) resultImage.Dispose();
			}
			foreach (Bitmap bitmap in bitmaps)
			{
				if (bitmap != null) bitmap.Dispose();
			}
			foreach (Bitmap resultBitmap in resultBitmaps)
			{
				if (resultBitmap != null) resultBitmap.Dispose();
			}
			images = new NGrayscaleImage[count];
			resultImages = new NGrayscaleImage[count];
			bitmaps = new Bitmap[count];
			resultBitmaps = new Bitmap[count];
			templates = new byte[count][];
			availableCount = 0;
			template = null;
			richTextBox.Clear();
			okButton.Enabled = false;
			LogWait();
		}

		private void OnEnrollModeChanged()
		{
			Clear();
		}

		private void OnShowIDChanged()
		{
			idLabel.Visible = idTextBox.Visible = showID;
		}

		private void OnImage(NGrayscaleImage image, string logText)
		{
			if (availableCount == count)
			{
				Clear();
			}
			else
			{
				ClearView();
			}
			LogLine(logText);
			LogLine("Size: {0}x{1}. Resolution: {2}x{3} dpi", image.Width, image.Height, image.HorzResolution, image.VertResolution);
			if (images[availableCount] != null) images[availableCount].Dispose();
			images[availableCount] = image;
			this.image = image;
			if (bitmaps[availableCount] != null) bitmaps[availableCount].Dispose();
			bitmaps[availableCount] = image.ToBitmap();
			nfView1.Image = bitmaps[availableCount];
			nfView2.Image = bitmaps[availableCount];
			OnImageChanged();

			UseWaitCursor = true;
			NGrayscaleImage resultImage = (NGrayscaleImage)image.Clone();
			Stopwatch sw = Stopwatch.StartNew();
			byte[] template;
			try
			{
				template = Data.VFExtractor.Extract(resultImage, NFPosition.Unknown, NFImpressionType.LiveScanPlain);
			}
			catch(Exception e)
			{
				string text = string.Format("Extraction error: {0}", e.Message);
				LogLine(text);
				MessageBox.Show(text, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			sw.Stop();
			UseWaitCursor = false;
			NFRecord templ = template == null ? null : new NFRecord(template);
			if (resultImages[availableCount] != null) resultImages[availableCount].Dispose();
			resultImages[availableCount] = resultImage;
			this.resultImage = resultImage;
			using (NImage ri = NImages.GetGrayscaleColorWrapper(resultImage, resultImageMinColor, resultImageMaxColor))
			{
				if (resultBitmaps[availableCount] != null) resultBitmaps[availableCount].Dispose();
				resultBitmaps[availableCount] = ri.ToBitmap();
			}
			templates[availableCount] = template;
			nfView2.ResultImage = resultBitmaps[availableCount];
			if (nfView2.Template != null) nfView2.Template.Dispose();
			nfView2.Template = templ;
			OnResultImageChanged();

			if (template == null)
			{
				string text = "Fingerprint image is of low quality";
				LogLine(text);
				MessageBox.Show(text, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			Settings settings = Settings.Default;
			if (settings.FPUseMinimalMinutiaCount && templ.Minutiae.Count < settings.FPMinimalMinutiaCount)
			{
				string text = string.Format("Fingerprint minutia count ({0}) is less than minimal value {1}", templ.Minutiae.Count, settings.FPMinimalMinutiaCount);
				LogLine(text);
				MessageBox.Show(text, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			LogLine("Template extracted{0}. G: {1}. Size: {2}. Time: {3}",
				Data.VFExtractor.UseQuality ? string.Format(". Quality: {0:P0}", Data.QualityToPercent(templ.Quality) / 100.0) : null,
				templ.G, Data.SizeToString(template.Length), Data.TimeToString(sw.Elapsed));
			availableCount++;

			if (availableCount == count)
			{
				if (enrollMode && settings.FPUseGeneralization)
				{
					Stopwatch swg = Stopwatch.StartNew();
					int baseTemplateIndex;
					try
					{
						template = Data.VFExtractor.Generalize(templates, out baseTemplateIndex);
					}
					catch (Exception e)
					{
						string text = string.Format("Generalization error: {0}", e.Message);
						LogLine(text);
						MessageBox.Show(text, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
						return;
					}
					swg.Stop();
					if (template == null)
					{
						string text = "Generalization failed";
						LogLine(text);
						MessageBox.Show(text, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
						return;
					}
					templ = new NFRecord(template);
					LogLine("Template generalized, based on image #{0}{1}. G: {2}. Size: {3}. Time {4}",
						baseTemplateIndex + 1,
						Data.VFExtractor.UseQuality ? string.Format(". Quality: {0:P0}", Data.QualityToPercent(templ.Quality) / 100.0) : null,
						templ.G, Data.SizeToString(template.Length), Data.TimeToString(swg.Elapsed));
					nfView1.Image = bitmaps[baseTemplateIndex];
					nfView2.Image = bitmaps[baseTemplateIndex];
					nfView2.ResultImage = resultBitmaps[baseTemplateIndex];
					nfView2.Template.Dispose();
					nfView2.Template = templ;
					this.image = images[baseTemplateIndex];
					this.resultImage = resultImages[baseTemplateIndex];
					OnImageChanged();
					OnResultImageChanged();
				}
				this.template = template;
				okButton.Enabled = true;
				if (showID && ID == string.Empty)
				{
					idTextBox.Select();
				}
				else
				{
					okButton.Select();
				}
			}
			else
			{
				LogWait();
			}
		}

		private void Log(string text, bool scroll)
		{
			richTextBox.AppendText(text);
			if (scroll) richTextBox.ScrollToCaret();
		}

		private void LogLine()
		{
			Log(Environment.NewLine, true);
		}

		private void Log(string text)
		{
			Log(text, true);
		}

		private void LogLine(string text)
		{
			Log(text, false);
			LogLine();
		}

		private void Log(string format, params object[] args)
		{
			Log(string.Format(format, args));
		}

		private void LogLine(string format, params object[] args)
		{
			LogLine(string.Format(format, args));
		}

		private void LogWait()
		{
			if (count == 1)
			{
				LogLine("Waiting for image");
			}
			else
			{
				LogLine("Waiting for image {0} of {1}", availableCount + 1, count);
			}
		}

		private void SaveImage(NGrayscaleImage image)
		{
			saveFileDialog.FileName = null;
			if (saveFileDialog.ShowDialog() == DialogResult.OK)
			{
				NImageFormat imageFormat = NImageFormat.Formats[saveFileDialog.FilterIndex - 1];
				string fileName = saveFileDialog.FileName;
				if (Path.GetExtension(fileName) == null)
				{
					fileName = fileName + '.' + imageFormat.DefaultFileExtension;
				}
				try
				{
					imageFormat.SaveImage(image, fileName);
				}
				catch (Exception ex)
				{
					MessageBox.Show(string.Format("Error saving to file \"{0}\": {1}", fileName, ex.Message),
						Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}

		private void SetShownImage(ShownImage value)
		{
			Settings.Default.FPShownImage = nfView2.ShownImage = value;
		}

		#endregion

		#region Public properties

		public bool EnrollMode
		{
			get
			{
				return enrollMode;
			}
			set
			{
				if (enrollMode != value)
				{
					enrollMode = value;
					OnEnrollModeChanged();
				}
			}
		}

		public bool ShowID
		{
			get
			{
				return showID;
			}
			set
			{
				if (showID != value)
				{
					showID = value;
					OnShowIDChanged();
				}
			}
		}

		public byte[] Template
		{
			get
			{
				return template;
			}
		}

		public string ID
		{
			get
			{
				return idTextBox.Text.Trim();
			}
			set
			{
				idTextBox.Text = value;
			}
		}

		#endregion

		private void openToolStripButton_Click(object sender, EventArgs e)
		{
			openFileDialog.FileName = null;
			if (openFileDialog.ShowDialog() == DialogResult.OK)
			{
				int formatIndex = openFileDialog.FilterIndex - 2;
				NImageFormat imageFormat =
					formatIndex == -1 || formatIndex == NImageFormat.Formats.Count
					? null : NImageFormat.Formats[formatIndex];
				string fileName = openFileDialog.FileName;

				NImage image;
				try
				{
					image = NImage.FromFile(fileName, imageFormat);
				}
				catch (Exception ex)
				{
					MessageBox.Show(string.Format("Error opening file \"{0}\": {1}", fileName, ex.Message),
						Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}

				Settings settings = Settings.Default;
				float horzResolution = image.HorzResolution;
				float vertResolution = image.VertResolution;
				if (!settings.PromptForImageResolution)
				{
					if (horzResolution < 250) horzResolution = 500;
					if (vertResolution < 250) vertResolution = 500;
				}
				else
				{
					ImageResolutionForm form = new ImageResolutionForm();
					form.HorzResolution = horzResolution;
					form.VertResolution = vertResolution;
					if (form.ShowDialog() != DialogResult.OK)
					{
						return;
					}
					horzResolution = form.HorzResolution;
					vertResolution = form.VertResolution;
				}
				NGrayscaleImage grayImage = (NGrayscaleImage)NImage.FromImage(NPixelFormat.Grayscale, 0, horzResolution, vertResolution, image);
				OnImage(grayImage, string.Format("Image loaded from file \"{0}\"", fileName));
			}
		}

		private void promptForResolutionToolStripButton_Click(object sender, EventArgs e)
		{
			Settings settings = Settings.Default;
			promptForResolutionToolStripButton.Checked = settings.PromptForImageResolution = !settings.PromptForImageResolution;
			settings.Save();
		}

		private void clearToolStripButton_Click(object sender, EventArgs e)
		{
			Clear();
		}

		private void NFRecordForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (scannerMan != null)
			{
				foreach(FPScanner scanner in scannerMan.Scanners)
				{
					scanner.StopCapturing();
				}
				scannerMan.Dispose();
			}
			Settings settings = Settings.Default;
			settings.NFRecordFromSize = WindowState != FormWindowState.Normal ? RestoreBounds.Size : Size;
			settings.NFRecordFormMaximized = WindowState == FormWindowState.Maximized;
			Settings.Default.Save();
		}

		private void NFRecordForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (DialogResult == DialogResult.OK)
			{
				if (showID && ID == string.Empty)
				{
					idTextBox.Select();
					MessageBox.Show("ID is not specified", Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
					e.Cancel = true;
				}
			}
		}

		private void saveToolStripSplitButton_ButtonClick(object sender, EventArgs e)
		{
			SaveImage(image);
		}

		private void saveImageToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveImage(image);
		}

		private void saveResultImageToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveImage(resultImage);
		}

		private void shownImageNoneToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SetShownImage(ShownImage.None);
		}

		private void shownImageOriginalToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SetShownImage(ShownImage.Original);
		}

		private void shownImageResultToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SetShownImage(ShownImage.Result);
		}

		private void showMinutiaeToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Settings.Default.FPShowMinutiae = nfView2.ShowMinutiae = !nfView2.ShowMinutiae;
		}

		private void NFRecordForm_Shown(object sender, EventArgs e)
		{
			if (Settings.Default.NFRecordFormMaximized) WindowState = FormWindowState.Maximized;
		}
	}
}
