/* $Id: demo.c 9136 2006-09-18 18:33:58Z grazvydas $ */
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <scanner.h>
#include <sys/time.h>
#include <pthread.h>
#include <glib.h>
#include <unistd.h>

#include "fp_engine.h"

#include "options.h"
#include "list.h"
#include "gtk_job.h"
#include "thread.h"


pthread_mutex_t	thr_mutex;
int con_mode;
static pthread_t scan_thread;
pthread_mutex_t scanner_muttex;
struct scanner_info *scanner = NULL;
struct scanner_info *scanner_list[] = { SCANNERS NULL };
static void *g_record_buffers[3] = { NULL, NULL, NULL };
static NSizeType g_max_record_size;
HVFExtractor vf_extractor;
HVFMatcher vf_matcher;

void timer_start (struct timeval *time)
{
	gettimeofday (time, NULL);
}

long timer_end (struct timeval *start)
{
	struct timeval end;
	long time;

	gettimeofday (&end, NULL);

	time = end.tv_usec - start->tv_usec +
		(end.tv_sec - start->tv_sec) * 1000000;

	return time;
}

static void find_duplicates(void *record_buff, NSizeType record_size, NByte g_value)
{
	void		*record_item;
	NSizeType	record_item_size = 0;
	char		*name;
	NResult		result;
	int		size;

	result = VfmIdentifyStart(vf_matcher, record_buff, record_size, NULL);
	if (result != N_OK)
		return;

	if (options.use_g)
	{
		list_get_start_g(g_value);
		record_item = list_get_next_g(&name, &record_item_size);
	}
	else
	{
		list_get_start();
		record_item = list_get_next(&name, &record_item_size);
	}

	size = list_size();
	size = (size * options.match_at_most / 100) + 1;
	while (record_item != NULL && size--)
	{
		NInt score;

		result = VfmIdentifyNext(vf_matcher, record_item, record_item_size, NULL, &score);

		if (result != N_OK)
		{
			log_printf ("VfmIdentifyNext failed.\n");
			break;
		}

		if (score != 0)
		{
			log_printf ("Duplicate found: \"%s\", similarity: %d\n", name, score);
			break;
		}

		if (options.use_g)
			record_item = list_get_next_g(&name, &record_item_size);
		else
			record_item = list_get_next(&name, &record_item_size);
	}
	VfmIdentifyEnd(vf_matcher);
}

int generalize(unsigned char *img, int w, int h, int dpi, char *name, int mode)
{
	static int	count = 0;
	NResult		result;
	int		ret = 0;
	void		*gen_record_buffer;
	static NSizeType g_record_sizes[3];
	NSizeType	gen_record_size;
	NInt		base_template;
	HNFRecord	record;
	NInt		minutia_count;

	struct timeval	start_time;
	long		time;

	gen_record_buffer = malloc(g_max_record_size);
	if (gen_record_buffer == NULL)
	{
		return 1;
	}

	log_printf(">>>  Generalize (%d of %d) start  >>>\n", count+1, 3);
	timer_start(&start_time);

	result = VfeExtract(vf_extractor, w, h, w, dpi, dpi,
		img, nfpUnknown, nfitLiveScanPlain, g_record_buffers[count],
		g_max_record_size, &g_record_sizes[count]);

	time = timer_end(&start_time);

	log_printf("Extract time: %d.%06d sec.\n", time / 1000000, time % 1000000);
	if (result != N_OK)
	{
		log_printf ("Extraction failed: %d\n", result);
		goto gen_end;
	}

	if (g_record_sizes[count] == 0)
	{
		log_printf ("Low quality image.\n");
		goto gen_end;
	}

	NFRecordCreateFromMemory(g_record_buffers[count], g_record_sizes[count], 0, NULL, &record);
	NFRecordGetMinutiaCount(record, &minutia_count);
	paint_minutia(record, minutia_count);
	NFRecordFree(record);

	if (minutia_count < options.minutia_count)
		log_printf ("Minutia count too low (%d/%d)\n", options.minutia_count, minutia_count);
	else
		count ++;
	log_printf ("Image (%d of %d)\n", count, 3);

	if (count == 3)
	{
		NByte g_value;

		timer_start (&start_time);
		result = VfeGeneralize(vf_extractor, 3, (const void **)g_record_buffers, g_record_sizes,
			gen_record_buffer, g_max_record_size, &gen_record_size, &base_template);

		time = timer_end (&start_time);

		log_printf ("Generalization time: %d.%06d sec.\n", time / 1000000, time % 1000000);
		if (result == N_OK)
		{
			log_printf ("Generalized template is based on template number: %d\n", base_template+1);
		}
		else
		{
			log_printf ("Generalization failed: %d\n", result);
		}

		count = 0;

		if (result != N_OK)
			goto gen_end;

		name = prompt_name(name, mode);

		NFRecordCreateFromMemory(gen_record_buffer, gen_record_size, 0, NULL, &record);
		NFRecordGetG(record, &g_value);
		NFRecordFree(record);

		if (options.duplicates)
			find_duplicates(gen_record_buffer, gen_record_size, g_value);
		list_add(gen_record_buffer, gen_record_size, name, g_value);
	}

	ret = 1;
gen_end:
	free(gen_record_buffer);
	log_printf ("<<<  Generalize end  <<<\n");
	return ret;
}


int enroll(unsigned char *img, int w, int h, int dpi, char *name, int mode)
{
	NResult		result;
	NSizeType	record_size;
	int		ret = 0;
	void		*record_buffer = NULL;
	HNFRecord	record;
	NInt		minutia_count;
	NByte		g_value;

	struct timeval	start_time;
	long		time;

	VfeGetMaxTemplateSize(vf_extractor, &g_max_record_size);
	record_buffer = malloc(g_max_record_size);
	if (record_buffer == NULL)
	{
		return 1;
	}

	log_printf (">>>  Enrol start  >>>\n");
	timer_start (&start_time);

	result = VfeExtract(vf_extractor, w, h, w, dpi, dpi,
		img, nfpUnknown, nfitLiveScanPlain, record_buffer,
		g_max_record_size, &record_size);

	time = timer_end(&start_time);

	log_printf("Extraction time: %d.%06d sec.\n", time / 1000000, time % 1000000);
	if (result != N_OK)
	{
		log_printf("Extraction failed: %d\n", result);
		goto enr_end;
	}

	if (record_size == 0)
	{
		log_printf ("Low quality image.\n");
		goto enr_end;
	}

	NFRecordCreateFromMemory(record_buffer, record_size, 0, NULL, &record);
	NFRecordGetG(record, &g_value);
	NFRecordGetMinutiaCount(record, &minutia_count);

	log_printf("G: %d, size: %d bytes, minutia count: %d\n", g_value,
			record_size, minutia_count);

	paint_minutia(record, minutia_count);

	name = prompt_name(name, mode);
	if (minutia_count < options.minutia_count)
		log_printf ("Minutia count too low (%d/%d)\n", options.minutia_count, minutia_count);
	else
	{
		if (options.duplicates)
			find_duplicates(record_buffer, record_size, g_value);
		list_add(record_buffer, record_size, name, g_value);
	}

	NFRecordFree(record);
	ret = 1;
enr_end:
	free(record_buffer);
	log_printf ("<<<  Enrol end  <<<\n");
	return ret;
}


int identify(unsigned char *img, int w, int h, int dpi)
{
	NResult		result;
	int		ret = 0;
	NSizeType	record_size;
	void            *record_buffer = NULL;
	void		*record_item;
	NSizeType	record_item_size = 0;
	HNFRecord	record;

	char		*name;
	int		count = 0, matched = 0;
	NInt		minutia_count;
	int		size;
	NByte		g_value;

	struct timeval	start_time;
	long		time;

	VfeGetMaxTemplateSize(vf_extractor, &g_max_record_size);
	record_buffer = malloc(g_max_record_size);
	if (record_buffer == NULL)
	{
		return 1;
	}

	log_printf (">>>  Identify start\n");
	timer_start (&start_time);
	result = VfeExtract(vf_extractor, w, h, w, dpi, dpi,
		img, nfpUnknown, nfitLiveScanPlain, record_buffer,
		g_max_record_size, &record_size);
	time = timer_end (&start_time);

	log_printf ("Extraction time: %d.%06d sec.\n", time / 1000000, time % 1000000);
	if (result != N_OK)
	{
		log_printf ("Extraction failed: %d\n", result);
		goto iden_end;
	}

	if (record_size == 0)
	{
		log_printf ("Low quality image.\n");
		goto iden_end;
	}

	NFRecordCreateFromMemory(record_buffer, record_size, 0, NULL, &record);
	NFRecordGetG(record, &g_value);
	NFRecordGetMinutiaCount(record, &minutia_count);

	log_printf("G: %d, size: %d bytes, minutia count: %d\n", g_value,
			record_size, minutia_count);

	paint_minutia(record, minutia_count);

	if (minutia_count < options.minutia_count)
		log_printf ("Minutia count too low (%d/%d)\n", options.minutia_count, minutia_count);

	result = VfmIdentifyStart(vf_matcher, record_buffer, record_size, NULL);
	if (result != N_OK)
	{
		log_printf ("Identification start failed: %d\n", result);
		ret = 1;
		goto iden_end;
	}

	if (options.use_g)
	{
		list_get_start_g(g_value);
		record_item = list_get_next_g(&name, &record_item_size);
	}
	else
	{
		list_get_start();
		record_item = list_get_next(&name, &record_item_size);
	}

	time = 0;

	size = list_size();
	size = (size * options.match_at_most / 100) + 1;
	while (record_item != NULL && size--)
	{
		NInt score;
		timer_start(&start_time);
		result = VfmIdentifyNext(vf_matcher, record_item, record_item_size, NULL, &score);
		time += timer_end(&start_time);
		count ++;

		if (result != N_OK)
		{
			log_printf("VFIdentifyNext failed with %i\n", result);
			break;
		}
		if (score != 0)
		{
			matched++;
			log_printf ("Record \"%s\" similarity: %d\n", name, score);
		}
		else if (options.show_all_results)
		{
			log_printf ("Record \"%s\" did not match.\n", name);
		}
		if (score != 0 && !options.match_all_db)
			break;
		if (options.use_g)
			record_item = list_get_next_g(&name, &record_item_size);
		else
			record_item = list_get_next(&name, &record_item_size);
	}

	result = VfmIdentifyEnd(vf_matcher);
	if (result != N_OK)
	{
		log_printf ("Identification end failed: %d\n", result);
		ret = 1;
		goto iden_end;
	}


	if (count == 0)
	{
		ret = 1;
		goto iden_end;
	}
	log_printf ("Matching time: %d.%06d sec., checked: %d, matched: %i, database size: %d\n",
		time / 1000000, time % 1000000, count, matched, list_size());
	log_printf ("Effective matching speed: %d fingerprints per second\n",
		1000000 * count / time);

	ret = 1;
iden_end:
	free(record_buffer);
	log_printf ("<<<  Identify end  <<<\n");
	return ret;
}

static void *continues_scan_loop (void *none)
{
	int w, h, ret;
	char *image;

	while (1)
	{
		pthread_mutex_lock (&thr_mutex);
		pthread_mutex_unlock (&thr_mutex);

		if (scanner == NULL)
		{
			sleep (1);
			continue;
		}

		image = scanner_read (&w, &h);
		if (image != NULL)
		{
			gdk_threads_enter ();
			log_printf ("Scan complete\n");
			gdk_threads_leave ();

			gdk_threads_enter ();
			update_pixmap1 (image, w, h);
			gdk_threads_leave ();

			gdk_threads_enter ();
			switch (con_mode)
			{
				case 1:
					ret = enroll(image, w, h, scanner_dpi(),
							options.finger_id, 0);
					break;
				case 2:
					ret = identify(image, w, h, scanner_dpi());
					break;
				case 3:
					ret = generalize(image, w, h, scanner_dpi(),
							options.finger_id, 0);
					break;
			}
			gdk_threads_leave ();

			if (ret)
			{
				gdk_threads_enter ();
				update_pixmap2 (image, w, h, scanner_dpi());
				gdk_threads_leave();
			}
			while (scanner_read (&w, &h));
			gdk_threads_enter ();
			log_printf ("Finger up\n");
			gdk_threads_leave();
		}
		usleep (500);
	}
}


static void continues_scan_init (void)
{
	pthread_mutex_init (&thr_mutex, NULL);
	pthread_mutex_lock (&thr_mutex);
	pthread_mutex_init (&scanner_muttex, NULL);
	pthread_create (&scan_thread, NULL, continues_scan_loop, NULL);
}

int main (int argc, char *argv[])
{
	int ret, i;
	NResult result;

	g_thread_init (NULL);
	gdk_threads_init ();
	gdk_set_locale ();
	gtk_init (&argc, &argv);
	create_window ();

	result = VfeCreate(&vf_extractor);
	log_printf ("VFExtractor initialization: %d(%s)\n", result, (result == N_OK) ? "success" : "failed");

	result = VfmCreate(&vf_matcher);
	log_printf ("VFMatcher initialization: %d(%s)\n", result, (result == N_OK) ? "success" : "failed");

	VfeGetMaxTemplateSize(vf_extractor, &g_max_record_size);

	for (i = 0; i < 3; i++)
	{
		g_record_buffers[i] = malloc(g_max_record_size);
		if (g_record_buffers[i] == NULL)
		{
			log_printf("Failed to allocate buffers for generalize.");
		}
	}

	load_options ();

	set_title ();

	if (scanner)
	{
		ret = scanner_init ();
		log_printf ("scanner %s initialization: %d (%s)\n", scanner_name(), ret, (ret == 0) ? "success" : "failed");
	}

	continues_scan_init ();
	gtk_main ();

	VfmFree(vf_matcher);
	VfeFree(vf_extractor);

	for (i = 0; i < 3; i++)
	{
		free(g_record_buffers[i]);
	}

	return 0;
}

